/*
 * Created on 03 mai. 2004 by the Message Center Team
 *
 */
package mc.formgenerator.bonita;

import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.w3c.dom.Document;

//Bonita imports
import hero.interfaces.*;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSession;


/**
 * The aim of this class is to transform Bonita activity data to a xml file
 * @author sempereb
 */
public class BonitaActivityAdapter {




	/**
	 * Default class constructor 
	 */
	public BonitaActivityAdapter(){};
	
	
	

		
	/**
	 * Get the bonita properties for a specified activity as key value hash map
	 * @param theProjectName : The process concerned.
	 * @param theActivityName : The activity concerned.
	 * @return HashMap the table containing the properties.
	 */
	private HashMap getHBonitaData(String theProjectName, String theActivityName){
			
		//Hash map where 'Bonita key - value' will be put
		HashMap hMap = new HashMap();
			
		//The local home
		ProjectSessionHome projecth = null;
		
		//The bean
		ProjectSession project  = null;
		
		try{
			//Getting local home by using JNDI
			projecth = ProjectSessionUtil.getHome();
			
			//Project creation
			project = projecth.create();
					  
			//Project initialisation
			project.initProject(theProjectName);
			
			//We are now able to acess Bonita api and get properties for the concerned 
			Collection collectionActivityProperties = project.getNodeProperties(theActivityName);
			
			//Number of bean instances responding
			System.out.println( new String
				(new Integer(collectionActivityProperties.size()).toString()) + " Beans retrieved");
			
			//Property value of Bonita
			BnNodePropertyValue pv = null;
			
			//The current key of the property value object
			String currentKey = "";
			
			//The current value of the property value object
			String currentValue = "";

			//For all the keys we have to retrieve the associated value
			Iterator it = collectionActivityProperties.iterator();
			
			System.out.println();
			System.out.println("***Enterring Bonita API for activity "+theActivityName+"***********");
			System.out.println();
			
			while (it.hasNext()){
				//Getting the values
				pv = (BnNodePropertyValue)it.next();
			
				//Retrieve the key
				currentKey = pv.getTheKey();
				
				//And the value
				currentValue = pv.getTheValue();
				
				//Adding key and value in a hashMap
				hMap.put(currentKey, currentValue);
				
				//Display
				System.out.println(" Key: " + currentKey);
				System.out.println(" Value: " + currentValue);
				System.out.println();
			}
			
			System.out.println();
			System.out.println("***End of processing Bonita API for activity "+theActivityName+"***");
			System.out.println();

			//Returns the hash map
			return hMap;
		}
		catch(Exception e){
			System.out.println(e + " " + e.getMessage());
			return null;
		}
	}
	
	
	

	
	/**
	 * Provides a Document output for Bonita data
	 * @param theHashMap : The table containing the properties.
	 * @param theProjectName : The project concerned.
	 * @param theProjectName the activity concerned in the project
	 * @return Document document with the activity information : project name & activity name & activity properties.
	 * @throws IOExecption
	 */
	private Document generateBonitaDataDocument(HashMap theHashMap, String theProjectName, String theActivityName) 
		throws IOException{
		
		//DOM
		Document document = null;
		
		//a parser to generate the xml document from the dataProcess
		DocumentParser parser = new DocumentParser(); 
		
		//Create the dataProcess structure with bonita key value
		DataActivity data = new DataActivity(theProjectName, theActivityName, theHashMap);
			
		//Generate the xml document representing the dataProcess
		if( data != null )
			document = parser.createDocument(data);	
			
		return document;
	}
	
	
	
	
	/**
	 * Get the Bonita activity data identified by its name and put them in a DOM 
	 * @param theProjectName the name of the project we have to get process informations
	 * @param theActivityName the name of the activityname concerned
	 * @return the document that represents bonita activity data
	 * @throws LoginException
	 * @throws IOException
	 */
	public Document getActivityData(String theProjectName, String theActivityName) 
		throws IOException{
		
		//H map of Bonita data
		HashMap bonitaMap = new HashMap();
		
		//Set Bonita data in H map
		bonitaMap = this.getHBonitaData(theProjectName, theActivityName);
		
		//Document generated
		return this.generateBonitaDataDocument(bonitaMap, theProjectName, theActivityName);
	}
}


