/*
 * Created on 15 avr. 2004 by the Message Center Team
 *
 */
 
package mc.formgenerator.ggf.xslprocessing;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;

/**
 *  Execution of Xalan engine
 * 	@author sempereb
 */
public class XalanExecutor {

	//Path of the xml file
	public static String xmlFile;
	
	//Path of the xsl file
	public static String xslFile;
	
	//Path of the output file
	public static String outputFile;
	
	
	/**
	 * Default constructor of our class
	 */
	public XalanExecutor(){};




	/**
	 * Other constructor of our class
	 * It will create a XalanExecutor with a defined xsl file
	 * @param theXslFile: path of the xsl file
	 */
	public XalanExecutor(String theXslFile){
		//Set path of the xsl file
		xslFile = theXslFile;
	};




	/**
	 * Other execution with parameters
	 * @param theXmlFile: the source file
	 * @param theXslFile: the file that provides transformation
	 * @param theOutputFile: the file generated
	 * @throws TransformerException
	 * @throws TransformerConfigurationException
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public void XalanExecution(String theXmlFile, String theXslFile, String theOutputFile)
			throws TransformerException, TransformerConfigurationException, 
			FileNotFoundException, IOException{
		
		//Creation via the factory of an instance of Xalan transfomer
		TransformerFactory tFactory = TransformerFactory.newInstance();
	
		//We read the xsl file
		Transformer transformer = tFactory.newTransformer(new StreamSource( theXslFile ));

		//Transformation thanks to xalan for the xml file
		transformer.transform(new StreamSource( theXmlFile ), new StreamResult(new FileOutputStream( theOutputFile )));
	}




	/**
	 * Execution of Xalan on files in order to make dynamic XHTML 
	 * Here the output file is a Document instance
	 * @param document: the document to transform
	 * @return a Document that represents the document generated by the xsl engine
	 * @throws TransformerException
	 * @throws TransformerConfigurationException
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public Document XalanExecutionDOM(Document document)
			throws TransformerException, TransformerConfigurationException, 
			FileNotFoundException, IOException{
		
		//The DOM result
		DOMResult outputDOM = new DOMResult();
		
		//Creation via the factory of an instance of Xalan transfomer
		TransformerFactory tFactory = TransformerFactory.newInstance();
	
		//We read the xsl file
		Transformer transformer = tFactory.newTransformer(new StreamSource( xslFile ));

		//Transformation thanks to xalan for the xml file
		transformer.transform( new DOMSource(document), outputDOM);
		
		//Update of documentOutput 
		return (Document)outputDOM.getNode();
	}
}
