package hero.client.test;

/*
*
* NodeTests.java - 
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.interfaces.BnEdgeValue;
import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.BnNodeInterHookValue;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectHome;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.BnProjectValue;
import hero.interfaces.BnUser;
import hero.interfaces.BnUserHome;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserUtil;
import hero.interfaces.BnUserValue;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.UserSession;
import hero.interfaces.UserSessionHome;
import hero.interfaces.UserSessionUtil;
import hero.util.DuplicatedEdgeException;
import hero.util.HeroException;

import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;
import junit.framework.TestSuite;

public class ProjectSessionTests extends TestCase {
    private BnProjectHome projecth;
    private ProjectSessionHome projectSessionh;

    public ProjectSessionTests(String testname) {
	super(testname);
    }
    
    public static TestSuite suite() {
	return new TestSuite(ProjectSessionTests.class);
    }

    public void setUp() throws Exception {
    }

    public void testCreateProjectSession() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectsession");
	BnProjectValue pv=lProject.getDetails();
	BnUserLightValue[] uv=pv.getBnUsers();
	int i=0;
	while (i<uv.length) {
	    if (uv[i].getName().equals("admin"))
		break;
	    i++;
	}
	assertTrue("Admin is in the projet",i<uv.length);
    }

    public void testCloneProjectSession() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectsource");
	lProject.addNode("nclone",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("c1",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("c2",Constants.Nd.AND_JOIN_NODE);
	lProject.setNodeRole("nclone","admin");
	lProject.setNodeRole("c1","admin");
	lProject.setNodeRole("c2","admin");
	String edge1=lProject.addEdge("c1","c2");
	lProject = lHome.create();
	lProject.initProject("projectsource","tralara");	
	BnProjectValue pv=lProject.getDetails();
	assertTrue("Clone failure1",pv.getBnNodes().length==3);	  
	assertTrue("Clone failure2",pv.getName().equalsIgnoreCase("tralara"));	            
    }
    
   public void testImportProject() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectS");
	lProject.addNode("nclone",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("c1",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("c2",Constants.Nd.AND_JOIN_NODE);
	lProject.setNodeRole("nclone","admin");
	lProject.setNodeRole("c1","admin");
	lProject.setNodeRole("c2","admin");
	String edge1=lProject.addEdge("c1","c2");

	lProject = lHome.create();
	lProject.initProject("pDest");
	
	lProject.addNode("nclone2",Constants.Nd.AND_JOIN_NODE);
	lProject.setNodeRole("nclone2","admin");
	lProject.importProject("projectS","pDest");
	BnProjectValue pv=lProject.getDetails();
	assertTrue("Clone failure1",pv.getBnNodes().length==4);	  
	assertTrue("Clone failure2",pv.getName().equalsIgnoreCase("pDest"));	            
    }

    public void testAddUserToProject() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectadduser");	
	BnUserHome uHome = BnUserUtil.getHome();
	BnUserValue pdata=new BnUserValue();
	pdata.setName("bidochon");
	BnUser lUser = uHome.create(pdata);
	lProject.addUser("bidochon");
	pdata.setName("marcel");
	lUser = uHome.create(pdata);
	lProject.addUser("marcel");
	pdata.setName("germaine");
	lUser = uHome.create(pdata);
	lProject.addUser("germaine");
	assertTrue(lProject.containsUser("bidochon"));
    }

    public void testAddNodeToProject() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectaddnode");	
	lProject.addNode("thenode",1);
	BnNodeValue nv=lProject.getNodeValue("thenode");
	assertTrue("Creator not correctly set",nv.getCreator().getName().equals("admin"));
    }

    public void testAddEdgeToProject() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectaddedge");		
	lProject.addNode("thenode",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("thenode2",Constants.Nd.AND_JOIN_NODE);
	String ename=lProject.addEdge("thenode","thenode2");
	BnEdgeValue ev=lProject.getEdgeValue(ename);
	BnNodeValue nv=lProject.getNodeValue("thenode2");
	assertTrue("addEdge failed a1",ev.getState()==Constants.Ed.INITIAL);
	assertTrue("bad node state test3",nv.getState()==Constants.Nd.INITIAL);    }

    public void testProjectSessionAddUser() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("pst");		

	try {
	    pss.addUser("badUser");
	    assertTrue("Should have throw EJBException",false);
	} catch (Exception e) {
	    //expected behavior
	}
    }	

    public void testProjectSessionAddNode() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("pst2");	
	pss.addNode("test",Constants.Nd.AND_JOIN_NODE);
	BnNodeValue pv=pss.getNodeValue("test");

	assertTrue("Creator not correctly set",pv.getCreator().getName().equals("admin"));
    }

    public void testProjectSessionAddEdgesState() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("w");		

	pss.addNode("node1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("node2",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("node3",Constants.Nd.AND_JOIN_NODE);
	String edname=pss.addEdge("node1","node2");
	BnEdgeValue ev=pss.getEdgeValue(edname);
	BnNodeValue nv=pss.getNodeValue("node2");
	assertTrue("addEdge failed a1",ev.getState()==Constants.Ed.INITIAL);
	assertTrue("addEdge failed a1/n2",nv.getState()==Constants.Nd.INITIAL);
	edname=pss.addEdge("node2","node3");
	ev=pss.getEdgeValue(edname);
	BnNodeValue nv2=pss.getNodeValue("node2");
	BnNodeValue nv3=pss.getNodeValue("node3");
	assertTrue("addEdge failed a2",ev.getState()==Constants.Ed.INITIAL);
	assertTrue("addEdge failed a2/nv2",nv2.getState()==Constants.Nd.INITIAL);
	assertTrue("addEdge failed a2/nv3",nv3.getState()==Constants.Nd.INITIAL);
    }

    public void testcheckDuplicateEdge() throws Exception {
    
	try {
	    ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession pss = projectSessionh.create();
		pss.initProject("wde");		    
	    
	    pss.addNode("node1",Constants.Nd.AND_JOIN_NODE);
	    pss.addNode("node2",Constants.Nd.AND_JOIN_NODE);
	    pss.addEdge("node1","node2");
	    
	    pss.addNode("node3",Constants.Nd.AND_JOIN_NODE);
	    pss.addEdge("node2","node3");
	    
	    pss.addNode("node4",Constants.Nd.AND_JOIN_NODE);
	    pss.addEdge("node1","node4");
	    
	    pss.addEdge("node1","node2");
	    assertTrue("Should have thrown a DuplicatedEdgeException",false);
	}catch(DuplicatedEdgeException e){}
    }

    public void testDynamicProject() throws Exception {
	UserSessionHome usersh=UserSessionUtil.getHome();
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("dynamic");		

	pss.addNode("n1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n2",Constants.Nd.AND_JOIN_NODE);
	pss.addEdge("n1","n2");
	pss.setNodeRole("n1","admin");
	pss.setNodeRole("n1","admin");

	UserSession usr=usersh.create();

	usr.startActivity("dynamic","n1");

	pss.addNode("n3",Constants.Nd.AND_JOIN_NODE);
	String edge2=pss.addEdge("n2","n3");

	assertTrue("dynamic change failed - n1",pss.getNodeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2",pss.getNodeValue("n2").getState()==Constants.Nd.ANTICIPABLE);
	assertTrue("dynamic change failed - n3",pss.getNodeValue("n3").getState()==Constants.Nd.INITIAL);

	usr.startActivity("dynamic","n2");

	assertTrue("dynamic change failed - n1.1",pss.getNodeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2.1",pss.getNodeValue("n2").getState()==Constants.Nd.ANTICIPATING);
	assertTrue("dynamic change failed - n3.1",pss.getNodeValue("n3").getState()==Constants.Nd.ANTICIPABLE);

	pss.deleteEdge(edge2);

	assertTrue("dynamic change failed - n1.2",pss.getNodeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2.2",pss.getNodeValue("n2").getState()==Constants.Nd.ANTICIPATING);
	assertTrue("dynamic change failed - n3.2",pss.getNodeValue("n3").getState()==Constants.Nd.READY);

	pss.addEdge("n2","n3");

	assertTrue("dynamic change failed - n1.3",pss.getNodeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2.3",pss.getNodeValue("n2").getState()==Constants.Nd.ANTICIPATING);
	assertTrue("dynamic change failed - n3.3",pss.getNodeValue("n3").getState()==Constants.Nd.ANTICIPABLE);

    }
	
    public void TestDynamicProject2() throws Exception {
	UserSessionHome usersh=UserSessionUtil.getHome();
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();

	ProjectSession pss = projectSessionh.create();
	pss.initProject("dynamic2");	

	pss.addNode("n1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n2",Constants.Nd.AND_JOIN_NODE);
	String edge1=pss.addEdge("n1","n2");
	pss.setNodeRole("n1","admin");
	pss.setNodeRole("n2","admin");
	
	UserSession usr=usersh.create();

	usr.startActivity("dynamic2","n1");

	pss.addNode("n3",Constants.Nd.AND_JOIN_NODE);
	pss.addEdge("n2","n3");

	assertTrue("dynamic change failed - n1",pss.getEdgeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2",pss.getEdgeValue("n2").getState()==Constants.Nd.ANTICIPABLE);
	assertTrue("dynamic change failed - n3",pss.getEdgeValue("n3").getState()==Constants.Nd.INITIAL);

	pss.addNode("n4",Constants.Nd.AND_JOIN_NODE);
	pss.addEdge("n1","n4");
	pss.deleteEdge(edge1);

	assertTrue("dynamic change failed - n1.1",pss.getEdgeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2.1",pss.getEdgeValue("n2").getState()==Constants.Nd.READY);
	assertTrue("dynamic change failed - n3.1",pss.getEdgeValue("n3").getState()==Constants.Nd.INITIAL);
	assertTrue("dynamic change failed - n4.1",pss.getEdgeValue("n4").getState()==Constants.Nd.ANTICIPABLE);
    }

    public void testDeleteNode() throws Exception {
	UserSessionHome usersh=UserSessionUtil.getHome();
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	
	ProjectSession pss = projectSessionh.create();
	pss.initProject("deleteNode");		

	pss.addNode("n1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n2",Constants.Nd.AND_JOIN_NODE);
	String edge1=pss.addEdge("n1","n2");
	pss.setNodeRole("n1","admin");
	pss.setNodeRole("n2","admin");

	try {
	    pss.deleteNode("n2");
	} catch (HeroException he) { 
	    assertTrue("Deletion Failed",false); 
	}
	
	UserSession usr=usersh.create();

	usr.startActivity("deleteNode","n1");
	try {
	    pss.deleteNode("n1");
	    assertTrue("Deletion Should have Failed",false); 
	} catch (HeroException he) { // Normal Behavior
	}
    }	


    public void testPropertyNode() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");		

	pss.addNode("n1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n2",Constants.Nd.AND_JOIN_NODE);
	String edge1=pss.addEdge("n1","n2");
	pss.setNodeRole("n1","admin");
	pss.setNodeRole("n2","admin");
	pss.setNodeProperty("n1","kpn1","vpn1");
	pss.setNodeProperty("n2","kpn2","vpn2");
	pss.setNodeProperty("n2","kpn2","vpn22");
	
	pss.deleteNode("n2");
	
    }	

    public void testDeletePropertyNode() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");		

	pss.deleteNodeProperty("n1","kpn1");
	
    }	

    public void testPropertyProject() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");		

	pss.setProperty("maintenant","ara");
	pss.setProperty("deja","ahora");
	pss.setProperty("deja","ja");
	
    }	
    public void testDeletePropertyProject() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");	

	pss.deleteProperty("maintenant");	
    }	


    public void testNodePropertyPropagate() throws Exception {
	UserSessionHome usersh=UserSessionUtil.getHome();
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");	

	pss.addNode("n5",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n6",Constants.Nd.AND_JOIN_NODE);
	String edge1=pss.addEdge("n5","n6");
	pss.setNodeRole("n5","admin");
	pss.setNodeRole("n6","admin");
	pss.setNodeProperty("n5","value","15");
	pss.setNodeProperty("n5","value2","10");
	pss.setNodeProperty("n6","value","25");

	UserSession usr=usersh.create();
	usr.startActivity("propertyNode","n5");
	usr.terminateActivity("propertyNode","n5");
    }

  public void testGetPropertiesNode() throws Exception {

	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");	

	pss.addNode("p1",Constants.Nd.AND_JOIN_NODE);

	pss.setNodeProperty("p1","k1","10");
	
	Collection c=pss.getNodeProperties("p1");
        
	for(Iterator i=c.iterator();i.hasNext();){
	    BnNodePropertyValue p=(BnNodePropertyValue)i.next();
	    String key=p.getTheKey();
	    String value=p.getTheValue();
	    assertTrue("Get BnNode Properties",key.equalsIgnoreCase("k1"));
	    
	}
    }
  public void testGetProperties() throws Exception {

	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");		

	pss.setProperty("pk1","xxx");
	
	Collection c=pss.getProperties();
        
	for(Iterator i=c.iterator();i.hasNext();){
	    BnProjectPropertyValue p=(BnProjectPropertyValue)i.next();
	    String key=p.getTheKey();
	    String value=p.getTheValue();
	    if (value.equals("xxx"))
	    	assertTrue("Get Properties",key.equals("pk1"));
	}
    }

 public void testNodeDeadline() throws Exception {

	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("deadlineNode");	
		
	pss.addNode("d1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("d1","admin");
	pss.setNodeDeadline("d1",System.currentTimeMillis()+(long)10000);	
    }


   public void testJavaHook() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss=projectSessionh.create();
	pss.initProject("hookProject");
	pss.addHook("hero.hook.TestHook",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.JAVA);
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.addNodeHook("h1","hero.hook.TestHook",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.JAVA);
	pss.setNodeRole("h1","admin");

	usr.startActivity("hookProject","h1");
	usr.terminateActivity("hookProject","h1");	
	usr.startActivity("hookProject","h2");
	usr.terminateActivity("hookProject","h2");
	pss.deleteNodeHook("h1","hero.hook.TestHook");

   }

   public void testNodeInterHook() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("hookInterNode");		
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h1","admin");
	String edge1=pss.addEdge("h1","h2");
	String script =
	    "import hero.interfaces.BnProjectLocal;\n"
	    + "import hero.interfaces.BnNodeLocal;\n"
	    + "afterStart (Object b,Object n) {\n\n\n"
	    + "System.out.println(\"InteractiveBnNodee Hook test, node: \"+n.getName());"
	    + "}";
	pss.addNodeInterHook("h1","interTest",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	usr.startActivity("hookInterNode","h1");
	usr.terminateActivity("hookInterNode","h1");	
   }

   public void testNodeInterHookProperty() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
    UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("hookInterNodeP");		
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeProperty("h1","testp1","testv1",true);
	pss.setNodeProperty("h1","testp2","testv2",true);
	pss.setNodeRole("h1","admin");
	String edge1=pss.addEdge("h1","h2");
	String script =
	    "import hero.interfaces.BnProjectLocal;\n"
	    + "import hero.interfaces.BnNodeLocal;\n"
	    + "afterStart (Object b,Object n) {\n\n\n"
	    + "System.out.println(\"InteractiveBnNodee Hook properties tests, node: \"+n.getName());\n"
	    + "System.out.println(\"property \"+testp2);\n"
	    + "}";
	pss.addNodeInterHook("h1","interTest",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	usr.startActivity("hookInterNodeP","h1");
	usr.terminateActivity("hookInterNodeP","h1");
   }
    
    public void testProjectInterHook() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("hookInterProject");		
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h1","admin");
	String edge1=pss.addEdge("h1","h2");
	String script =
	    "import hero.interfaces.BnProjectLocal;\n"
	    + "import hero.interfaces.BnNodeLocal;\n"
	    + "afterStart (Object b,Object n) {\n\n\n"
	    + "System.out.println(\"InteractiveBnProjectt Hook test, node: \"+n.getName());"
	    + "}";
	pss.addInterHook("projectInterTest",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	usr.startActivity("hookInterProject","h1");
	usr.terminateActivity("hookInterProject","h1");	
   }

    public void testDuplicatedInterHook() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("dupliHookInter");		
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h1","admin");
	String edge1=pss.addEdge("h1","h2");
	String script =
	    "import hero.interfaces.BnProjectLocal;\n"
	    + "import hero.interfaces.BnNodeLocal;\n"
	    + "afterStart (Object b,Object n) {\n\n\n"
	    + "System.out.println(\"InteractiveBnProjectt Hook test, node: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
	    + "}";
	pss.addInterHook("projectInterTest",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	try {
	    pss.addInterHook("projectInterTest",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	    assertTrue("should have failed", false);
	} catch (HeroException h) {
	    // normal behavior
	}
	usr.startActivity("dupliHookInter","h1");
	usr.terminateActivity("dupliHookInter","h1");	
   }
    
   public void testSubProcess() throws Exception {

	  ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	  ProjectSession pss = projectSessionh.create();
	  pss.initProject("SubProject");
	  pss.addNode("subNode1",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("subNode2",Constants.Nd.AND_JOIN_NODE);
	  pss.addEdge("subNode1","subNode2");	
	  
	  ProjectSession pss1 = projectSessionh.create();
	  pss1 = projectSessionh.create();
	  pss1.initProject("ParentProject");	
	  pss1.addNode("node1",Constants.Nd.AND_JOIN_NODE);
	  pss1.addNode("node2",Constants.Nd.AND_JOIN_NODE);
	  pss1.addNode("SubProject",Constants.Nd.SUB_PROCESS_NODE);
	  pss1.addEdge("node1","SubProject");
	  pss1.addEdge("SubProject","node2");
	  UserSessionHome usersh=UserSessionUtil.getHome();
	  UserSession usr=usersh.create();
	  usr.startActivity("ParentProject","node1");
	  usr.startActivity("ParentProject","SubProject");
	  usr.terminateActivity("ParentProject","node1");
	  try {
		usr.terminateActivity("ParentProject","SubProject");
	  } catch (Exception he) { 
		assertTrue("SubProcess termination failed",true); 
	  }	
	  assertTrue("SubProcess Error", pss1.getNodeState("SubProject")==Constants.Nd.EXECUTING);
	
	  // Terminate all SubProcess activities
	  
	  usr.startActivity("SubProject","subNode1");
	  usr.startActivity("SubProject","subNode2");
	  usr.terminateActivity("SubProject","subNode1");
	  usr.terminateActivity("SubProject","subNode2"); 
	  usr.terminateActivity("ParentProject","SubProject");	
	  assertTrue("SubProcess Error", pss1.getNodeState("SubProject")==Constants.Nd.TERMINATED);
	  }
	  
	public void testSimpleInstantiation() throws Exception {

	  ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	  ProjectSession pss = projectSessionh.create();
	  pss.initProject("Instantiation");
	  pss.addNode("node1",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("node2",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("node3",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("node4",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
	  pss.addEdge("node1","node2");
	  pss.addEdge("node2","node3");
	  pss.addEdge("node2","node4");
	  String script =
			"import hero.interfaces.BnProjectLocal;\n"
			+ "import hero.interfaces.BnNodeLocal;\n"
			+ "afterStart (Object b,Object n) {\n\n\n"
			+ "System.out.println(\"Instantiation test, node: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
			+ "}";
	  pss.addNodeInterHook("node2","node2",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	  pss.addUser("admin2");
	  pss.setUserRole("admin2","InitialRole");
	  pss.setNodeRole("node1","admin");
	  
	  ProjectSession pss1 = projectSessionh.create();
	  pss1.instantiateProject("Instantiation");
	  assertTrue("Instantiation Error, instance name error", pss1.getName().equals("Instantiation_instance1"));
	  assertTrue("Instantiation Error, role error in node1", pss1.getNodeRoleName("node1").equals("admin"));
	  Collection users = pss1.getUsers();
	  Collection hooks = pss1.getNodeInterHooks("node2");
	  Iterator i = hooks.iterator();
	  assertTrue("Instantiation Error, hook does not exist", (((BnNodeInterHookValue)i.next()).getName()).equals("node2"));
	  assertTrue("Instantiation Error, user admin2 does not exist", users.contains("admin2"));
	  assertTrue("Instantiation Error, type error in node1", pss1.getNodeType("node1")== Constants.Nd.AND_JOIN_NODE);
	  assertTrue("Instantiation Error, type error in node4", pss1.getNodeType("node4")== Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
	  try {
		pss1.addInterHook("node2",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	  } catch (Exception he) { 
		  assertTrue("Instantiation error when copy interHooks in node2",true); 
	  }	
	}
	
	public void testSubProcessInstantiation() throws Exception {

	  ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	  
	  ProjectSession psSub = projectSessionh.create();
	  psSub.initProject("node5");
	  psSub.addNode("subNode1",Constants.Nd.AND_JOIN_NODE);
	  psSub.addNode("subNode2",Constants.Nd.AND_JOIN_NODE);
	  psSub.addEdge("subNode1","subNode2");
	  
	  String script =
			"import hero.interfaces.BnProjectLocal;\n"
			+ "import hero.interfaces.BnNodeLocal;\n"
			+ "afterStart (Object b,Object n) {\n\n\n"
			+ "System.out.println(\"SPInstantiation test, SubProcess: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
			+ "}";
	  psSub.addNodeInterHook("subNode2","subNode2",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	  psSub.addUser("admin2");
	  psSub.setNodeRole("subNode1","admin");
	  
	  ProjectSession pss = projectSessionh.create();
	  pss.initProject("SPInstantiation");
	  pss.addNode("node1",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("node2",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("node3",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("node4",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
	  pss.addNodeSubProcess("node5","SPInstantiation");
	  pss.addEdge("node1","node2");
	  pss.addEdge("node2","node3");
	  pss.addEdge("node2","node4");
	  pss.addEdge("node3","node5");
	  script =
			"import hero.interfaces.BnProjectLocal;\n"
			+ "import hero.interfaces.BnNodeLocal;\n"
	  		+ "afterStart (Object b,Object n) {\n\n\n"
			+ "System.out.println(\"SPInstantiation test, node: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
			+ "}";
	 pss.addNodeInterHook("node2","node2",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	 pss.addUser("admin2");
	 pss.setUserRole("admin2","InitialRole");
	 pss.setNodeRole("node1","admin");
	 ProjectSession pss1 = projectSessionh.create();
	 pss1.instantiateProject("SPInstantiation");
	 assertTrue("Instantiation Error, instance name error", pss1.getName().equals("SPInstantiation_instance1"));
	 assertTrue("Instantiation Error, role error in node1", pss1.getNodeRoleName("node1").equals("admin"));
	 Collection users = pss1.getUsers();
	 assertTrue("Instantiation Error, user admin2 does not exist", users.contains("admin2"));
	 assertTrue("Instantiation Error, type error in node1", pss1.getNodeType("node1")== Constants.Nd.AND_JOIN_NODE);
	 assertTrue("Instantiation Error, type error in node4", pss1.getNodeType("node4")== Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
	 try {
		pss1.addNodeInterHook("node2","node2",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
     } catch (Exception he) { 
	    assertTrue("Instantiation error when copy interHooks in node2",true); 
	 }	
	}
	
	public void testGetProjectNameOfInstance() throws Exception {
	    ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	    ProjectSession ps = projectSessionh.create();
		ps.initProject("ofInstance");
		ProjectSession pss1 = projectSessionh.create();
		String instanceName = pss1.instantiateProject("ofInstance");
		String projectName = pss1.getProjectNameOfInstance(instanceName);
		assertTrue("Error in getProjectNameOfInstance",projectName.equals("ofInstance"));
		try {
			projectName = pss1.getProjectNameOfInstance("ofInstance");
		 } catch (Exception io) { 
			assertTrue("getProjectNameOfInstance error when is not a valid instance",true); 
		 }		
	}
	
	public void testGetParent() throws Exception {
		// Test when we creates a subProcess activity within a project
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initProject("getParent");
		ProjectSession ps2 = projectSessionh.create();
		ps2.initProject("getParent2");
		ps.addNode("getParent2",Constants.Nd.SUB_PROCESS_NODE);
		String p= ps2.getParent();
		assertTrue("Error in getParent when add subProcess",p.equals("getParent"));
		
		// Test when we creates an instance of a project
		String instance=ps.instantiateProject("getParent");
		String pInstance=ps.getParent();
		assertTrue("Error in getParent when instantiate",pInstance.equals("getParent_instance1"));
		
	}
	
	public void testUnsetUserRole() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initProject("unset");
		ProjectSession ps2 = projectSessionh.create();
		ps2.initProject("unset2");
		
		ps.addUser("admin2");
		ps.setUserRole("admin2","InitialRole");
		ps2.addUser("admin2");
		ps2.setUserRole("admin2","InitialRole");

		ps.unsetUserRole("admin2","InitialRole");
		Collection names = ps.getUserRolesInProjectNames("admin2");
		
		assertTrue("Error in unset user roles: InitialRole",!names.contains("InitialRole"));
		assertTrue("Error in unset user roles: admin",names.contains("admin2"));
	}
	
	public void testIteration() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initProject("Iteration");
		
		// add Nodes
		ps.addNode("node0",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node1",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node2",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node3",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node4",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node5",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node6",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node7",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node8",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.setNodeTraditional("node8");
		ps.addNode("node9",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.setNodeTraditional("node9");

		// add Edges

		ps.addEdge("node0","node1");
		ps.addEdge("node1","node3");
		ps.addEdge("node1","node4");
		ps.addEdge("node1","node5");
		ps.addEdge("node3","node6");
		ps.addEdge("node4","node6");
		ps.addEdge("node4","node2");
		ps.addEdge("node5","node6");
		ps.addEdge("node6","node7");
		ps.addEdge("node3","node8");
		ps.addEdge("node8","node9");
		ps.addEdge("node5","node4");
		
		// set Iteration
		ps.addIteration("node6","node1","iterate.equals(\"true\")");
		
		// Set user role
		ps.setUserRole("admin","InitialRole");
		// Instantiate project
		String project = ps.instantiateProject("Iteration");
		ps.setProperty("iterate","true");
		
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();
		usr.startActivity(project,"node0");
				
		assertTrue("Error in Iterate, startActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in Iterate, startActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.ANTICIPABLE);
		
		usr.terminateActivity(project,"node0");
		
		assertTrue("Error in Iterate, terminateActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in Iterate, terminateActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.READY);
		int index = 0;
		while (index<2)
		{
			usr.startActivity(project,"node1");		
			assertTrue("Error in Iterate, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
			assertTrue("Error in Iterate, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.ANTICIPABLE);
			assertTrue("Error in Iterate, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in Iterate, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.ANTICIPABLE);
			
			usr.terminateActivity(project,"node1");		
			assertTrue("Error in Iterate, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);			
			assertTrue("Error in Iterate, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.READY);
			assertTrue("Error in Iterate, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in Iterate, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
			
			usr.startActivity(project,"node3");
			usr.startActivity(project,"node5");
			usr.startActivity(project,"node4");
			try{
				usr.startActivity(project,"node2");
				assertTrue("Error in Iterate, startActivity node2: node2 should be failed",false);	
			}catch(Exception e){}
			
			usr.terminateActivity(project,"node3");
			usr.terminateActivity(project,"node5");
			usr.terminateActivity(project,"node4");
			
			assertTrue("Error in Iterate, terminateActivity node3: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.INITIAL);
			usr.startActivity(project,"node6");
			usr.terminateActivity(project,"node6");
			assertTrue("Error in Iterate, terminateActivity node6: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.INITIAL);
			index++;
		}
		
		usr.startActivity(project,"node1");		
		assertTrue("Error in Iterate, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in Iterate, startActivity node1: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in Iterate, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.ANTICIPABLE);
		assertTrue("Error in Iterate, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in Iterate, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.ANTICIPABLE);
			
		usr.terminateActivity(project,"node1");		
		assertTrue("Error in Iterate, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in Iterate, terminateActivity node1: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in Iterate, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.READY);
		assertTrue("Error in Iterate, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in Iterate, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
			
		try{
			usr.startActivity(project,"node2");
			assertTrue("Error in Iterate, startActivity node2: node2 should be failed",false);	
		}catch(Exception e){}
		usr.startActivity(project,"node3");
		usr.startActivity(project,"node5");
		usr.startActivity(project,"node4");
		
		usr.terminateActivity(project,"node3");
		usr.terminateActivity(project,"node5");
		usr.terminateActivity(project,"node4");
			
		assertTrue("Error in Iterate, terminateActivity node3: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.INITIAL);
		usr.startActivity(project,"node6");
		
		ps.setProperty("iterate","false");
		
		usr.terminateActivity(project,"node6");
		assertTrue("Error in Iterate, terminateActivity node6: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.TERMINATED);
		
		assertTrue("Error in Iterate, terminateActivity node6, after iterate: node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.READY);
		assertTrue("Error in Iterate, terminateActivity node6, after iterate: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.READY);
		assertTrue("Error in Iterate, terminateActivity node6, after iterate: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in Iterate, terminateActivity node6, after iterate: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.TERMINATED);
		
	}
	
		
}
