package hero.session;
/*
* 01/02/2004 - 17:04:13
*
* ProjectSessionEJB.java - 
* Copyright (C) 2004 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
import hero.interfaces.BnAgentEdgeLocal;
import hero.interfaces.BnAgentLocal;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnNodeInterHookLocal;
import hero.interfaces.BnNodeHookLocal;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnIterationLightValue;
import hero.interfaces.BnProjectInterHookLocal;
import hero.interfaces.BnProjectHookLocal;
import hero.interfaces.BnProjectPropertyLocal;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.ProjectSessionLocal;
import hero.interfaces.ProjectSessionLocalHome;
import hero.interfaces.ProjectSessionUtil;
import hero.util.EngineException;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.HeroHookException;
import hero.interfaces.JMSServicesSessionUtil;
import hero.interfaces.JMSServicesSessionLocalHome;
import hero.interfaces.JMSServicesSessionLocal;

import hero.interfaces.BnNodePerformerAssignLocal;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.util.MailNotification;
import hero.interfaces.Constants;

import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import org.apache.log4j.Logger;

import bsh.Interpreter;

/**
 * Session Bean Template
 *
 * @ejb:bean name="Engine"
 *           display-name="Engine Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/Engine"
 *           local-jndi-name="ejb/hero/Engine_L"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 * @ejb.permission role-name="Admin,user"
 * @ejb:transaction type="Required"
 * @jonas.bean
 *      ejb-name="Engine"
 *      jndi-name="ejb/hero/Engine"
 **/

public class EngineBean implements SessionBean, Constants {

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Logger log = Logger.getLogger("log");

	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	private BnProjectLocal mProject;
	private ProjectSessionLocal mPSession;
	private JMSServicesSessionLocal jms;
	private boolean raiseError = false;

	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------  
	/**
	 * Terminates the project
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void terminate(String projectName)
		throws EngineException, HeroException {
		log.debug("Start terminate the project: '" + projectName + "'");
		BnProjectLocalHome pHome;
		BnProjectLocal pLocal;
		try {
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			pLocal = pHome.findByName(projectName);
			if (nodesTerminated(pLocal.getBnNodes())) {
				pLocal.setState(hero.interfaces.Constants.Pj.TERMINATED);
			} else {
				throw new EngineException("Project cannot be terminated. Some nodes are still active");
			}
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		}
		log.debug(logProjectProperties("Properties after terminated: "));
		log.debug("End terminate the project: '" + projectName + "'");
	}
	
	/**
		 * Starts the activity nodeName
		 * @param nodeName the name of the activity
		 * @ejb:interface-method view-type="both"
		 *
		 **/
		public void startActivity(String projectName, String nodeName)
			throws EngineException, HeroException {
			log.debug("start the Activity '" + nodeName +"'");
			ProjectSessionLocalHome psHome;
			BnProjectLocalHome pHome;
			try {
				psHome = ProjectSessionUtil.getLocalHome();
				pHome = BnProjectUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
			}
			try {
				mProject = pHome.findByName(projectName);
				mPSession = psHome.create();
				mPSession.initProject(projectName);
			} catch (FinderException fe) {
				throw new HeroException("Project does not exist " + projectName);
			} catch (CreateException he) {
				throw new HeroException("ProjectSession Error" + projectName);
			}
			log.debug(logActivityProperties(nodeName,"Properties before start:"));
			BnNodeLocal nd = mProject.getBnNode(nodeName);
			// if activity performer != null and user == activity performer --> performer assigment case
			// if activity performer is null --> common case
			if ( (nd.getActivityPerformer() != null  && ( nd.getActivityPerformer()).equals(mContext.getCallerPrincipal().getName())) || ! (nd.getActivityPerformer() != null))
			{
				execute(nodeName, hero.interfaces.Constants.Nd.START);
				if (mProject.getState()== hero.interfaces.Constants.Pj.INITIAL)
					mProject.setState(hero.interfaces.Constants.Pj.STARTED);
				if (raiseError == true)
				{
					raiseError = false;
					throw new HeroHookException("An error is produced during the execution of a BeforeStart hook...");
				}
			}
			else
			   throw new HeroException("Only activity performer can execute this activity.");
				
			log.debug("End of start activity '" + nodeName + "'");
			log.debug(logActivityProperties(nodeName, "Properties after start:"));

		}

		/**
		 * Starts the activity nodeName
		 * @param nodeName the name of the activity
		 * @ejb:interface-method view-type="both"
		 *
		 **/
		public void terminateActivity(String projectName, String nodeName)
			throws EngineException, HeroException {
			log.debug("Start terminate the activity: '" + nodeName + "'");
			ProjectSessionLocalHome psHome;
			BnProjectLocalHome pHome;
			try {
				psHome = ProjectSessionUtil.getLocalHome();
				pHome = BnProjectUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
			}
			try {
				mProject = pHome.findByName(projectName);
				mPSession = psHome.create();
				mPSession.initProject(projectName);
			} catch (FinderException fe) {
				throw new HeroException("Project does not exist " + projectName);
			} catch (CreateException he) {
				throw new HeroException("ProjectSession Error" + projectName);
			}
			log.debug(logActivityProperties(nodeName,"Properties before terminate:"));
			execute(nodeName, hero.interfaces.Constants.Nd.TERMINATE);
			if (raiseError == true)
			{
				raiseError = false;
				throw new HeroHookException("An error is produced during the execution of a afterTerminate hook...");
			}
			log.debug("End terminate the activity: '" + nodeName + "'");
			log.debug(logActivityProperties(nodeName,"Properties after terminate:"));
		}

		/**
		 * Starts the activity nodeName
		 * @param nodeName the name of the activity
		 * @ejb:interface-method view-type="both"
		 *
		 **/
		public void cancelActivity(String projectName, String nodeName)
			throws EngineException, HeroException {
		
			ProjectSessionLocalHome psHome;
			BnProjectLocalHome pHome;
			try {
				psHome = ProjectSessionUtil.getLocalHome();
				pHome = BnProjectUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
			}
			try {
				mProject = pHome.findByName(projectName);
				mPSession = psHome.create();
				mPSession.initProject(projectName);
			} catch (FinderException fe) {
				throw new HeroException("Project does not exist " + projectName);
			} catch (CreateException he) {
				throw new HeroException("ProjectSession Error" + projectName);
			}
			execute(nodeName, hero.interfaces.Constants.Nd.CANCEL);
			log.debug("cancel the activity: '" + nodeName+ "'");
		}

		/**
		 * Suspend the activity nodeName
		 * @param nodeName the name of the activity
		 * @ejb:interface-method view-type="both"
		 *
		 **/
		public void suspendActivity(String projectName, String nodeName)
			throws EngineException, HeroException {
			//log.debug("suspendActivity: nodeName=" + nodeName);
			ProjectSessionLocalHome psHome;
			BnProjectLocalHome pHome;
			try {
				psHome = ProjectSessionUtil.getLocalHome();
				pHome = BnProjectUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
			}
			try {
				mProject = pHome.findByName(projectName);
				mPSession = psHome.create();
				mPSession.initProject(projectName);
			} catch (FinderException fe) {
				throw new HeroException("Project does not exist " + projectName);
			} catch (CreateException he) {
				throw new HeroException("ProjectSession Error" + projectName);
			}
			execute(nodeName, hero.interfaces.Constants.Nd.SUSPEND);
		}

		/**
		 * Non_Automatic operation method that resume specify activity
		 */

		/**
		 * Resume the activity nodeName
		 * @param nodeName the name of the activity
		 * @ejb:interface-method view-type="both"
		 *
		 **/
		public void resumeActivity(String projectName, String nodeName)
			throws EngineException, HeroException {
			//log.debug("resumeActivity: nodeName=" + nodeName);
			ProjectSessionLocalHome psHome;
			BnProjectLocalHome pHome;
			try {
				psHome = ProjectSessionUtil.getLocalHome();
				pHome = BnProjectUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
			}
			try {
				mProject = pHome.findByName(projectName);
				mPSession = psHome.create();
				mPSession.initProject(projectName);
			} catch (FinderException fe) {
				throw new HeroException("Project does not exist " + projectName);
			} catch (CreateException he) {
				throw new HeroException("ProjectSession Error" + projectName);
			}
			execute(nodeName, hero.interfaces.Constants.Nd.RESUME);
		}

		/**
		 * Active an External Agent
		 * @param agentName the name of the external agent
		 * @ejb:interface-method view-type="both"
		 *
		 **/
		public void activeAgent(String projectName, String agentName)
			throws EngineException, HeroException {
			//log.debug("activeAgent: agentName=" + agentName);
			BnAgentLocal ag;
			ProjectSessionLocalHome psHome;
			BnProjectLocalHome pHome;
			try {
				psHome = ProjectSessionUtil.getLocalHome();
				pHome = BnProjectUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
			}
			try {
				mProject = pHome.findByName(projectName);
				mPSession = psHome.create();
				mPSession.initProject(projectName);
				ag = mProject.getBnAgent(agentName);
				mPSession.setAgentState(ag, hero.interfaces.Constants.Ag.ACTIVE);
				Collection edges = ag.getOutEdges();
				if (!edges.isEmpty())
					modifyAgentEdges(edges);
			} catch (FinderException fe) {
				throw new HeroException("Project does not exist " + projectName);
			} catch (CreateException he) {
				throw new HeroException("ProjectSession Error" + projectName);
			}
		}

	// Utility methods

	private void execute(String _node, int operation) throws HeroException, EngineException {
		log.debug("execute: node="+ _node+" operation="+ hero.interfaces.Constants.Nd.operationName[operation]);
		BnNodeLocal nd;
		nd = mProject.getBnNode(_node);

		int newState = nd.getTransition().computeState(nd, operation);
		log.debug("Execute: New node State "+ _node+ " :"+ hero.interfaces.Constants.Nd.nodeStateName[newState]);

		if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION)
			throw new EngineException("Bad state transition " + _node);

		if (newState != nd.getState()) {
			if (newState == hero.interfaces.Constants.Nd.TERMINATED) {
				if (nd.getType()== hero.interfaces.Constants.Nd.SUB_PROCESS_NODE)
				{
				  this.terminate(_node);
				  mPSession.propagatesSubProcessProperties(_node);
			    }
				jms.sendProjectEvent(EventConstants.TERMINATED,mProject.getName(),_node,mContext.getCallerPrincipal().getName());

				//this.hookExecute(_node, hero.interfaces.Constants.Nd.EXECUTE);
				this.hookExecute(_node,	hero.interfaces.Constants.Nd.BEFORETERMINATE);
				mPSession.propagateNodeProperties(_node);
			} else if (newState == hero.interfaces.Constants.Nd.EXECUTING) {
				if (nd.getType()== hero.interfaces.Constants.Nd.SUB_PROCESS_NODE)
				   mPSession.propagatesParentProperties(_node);
				jms.sendProjectEvent(EventConstants.START,mProject.getName(),_node,mContext.getCallerPrincipal().getName());
				this.hookExecute(_node,	hero.interfaces.Constants.Nd.BEFORESTART);
			} else if (newState == hero.interfaces.Constants.Nd.ANTICIPATING) {
				jms.sendProjectEvent(EventConstants.ANTICIPATING,mProject.getName(),_node,mContext.getCallerPrincipal().getName());
				this.hookExecute(_node,hero.interfaces.Constants.Nd.ANTICIPATE);
			} else if (newState == hero.interfaces.Constants.Nd.DEAD) {
				jms.sendProjectEvent(EventConstants.CANCELED,mProject.getName(),_node,mContext.getCallerPrincipal().getName());
				this.hookExecute(_node, hero.interfaces.Constants.Nd.ONCANCEL);
			}

			if (activityStarted(newState)) {
				this.hookExecute(_node,hero.interfaces.Constants.Nd.AFTERSTART);
			}
			
			// Activities iteration
			if (newState == hero.interfaces.Constants.Nd.TERMINATED && mPSession.getIterationExist(nd.getName()))
			{
				mPSession.setNodeState(nd,hero.interfaces.Constants.Nd.INITIAL);
				String preNode = mPSession.getToIteration(nd.getName());
				String condition = mPSession.getIterationCondition(nd.getName());
	
				if (preNode != null && (evaluateCondition(condition, nd))==Constants.CONDITION_TRUE)
				{
					Collection nodes = getWay(nd.getName(),preNode);
					Iterator it = nodes.iterator();
					while(it.hasNext())
					    mPSession.setNodeState(mProject.getBnNode((String)it.next()),hero.interfaces.Constants.Nd.INITIAL);		 					BnNodeLocal preNodeLocal = mProject.getBnNode(preNode);
					mPSession.setNodeState(mProject.getBnNode(preNode),hero.interfaces.Constants.Nd.READY);
				        if ( preNodeLocal.getBnNodePerformerAssign() != null)
   				           this.performAssign(preNodeLocal);
				}
				else if (preNode != null)
				{
					mPSession.setNodeState(nd,hero.interfaces.Constants.Nd.TERMINATED);
					activeStandByEdges(nd.getName(),preNode);
				}
			}
			else
			{
			  mPSession.setNodeState(nd, newState);
			  mPSession.setNodeExecutor(_node);
			  
			  try{
				 if (newState==hero.interfaces.Constants.Nd.READY && nd.getBnNodePerformerAssign() != null)
				   this.performAssign(nd);			  	
			   } catch (Exception e){
				 e.printStackTrace();
				 throw new HeroException("Error when call of performAssign()!!!");
			   }
			  Collection edges = nd.getOutBnEdges();
			  if (!edges.isEmpty())
			  	modifyNodeEdges(newState, edges);
			}
		}
				
		if (newState == hero.interfaces.Constants.Nd.TERMINATED)
			this.hookExecute(_node,	hero.interfaces.Constants.Nd.AFTERTERMINATE);
			
		if (newState == hero.interfaces.Constants.Nd.EXECUTING && nd.getType() == hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE)
			this.execute(_node,hero.interfaces.Constants.Nd.TERMINATE);
	}

	private void performAssign(BnNodeLocal nd) throws  HeroException {
	   try {	
		   BnNodePerformerAssignLocal performerAssign = null;
		   BnUserLocal mUser;
		   String creator = mContext.getCallerPrincipal().getName();
		   performerAssign = nd.getBnNodePerformerAssign();
		   hero.performerAssign.PerformerAssign h = hero.performerAssign.PerformerAssign.make(performerAssign.getName(), performerAssign.getType());
		   h.execute(this, performerAssign.getType(), nd, creator);
		
		   // send notification to the activityPerformer of the node
		   String activityPerformerName = nd.getActivityPerformer();
		
		   try {
			   BnUserLocalHome userhome=hero.interfaces.BnUserUtil.getLocalHome();
			   mUser=userhome.findByName(activityPerformerName);
				
		  } catch (javax.naming.NamingException ne) {
			   throw new HeroException(ne.getMessage());
		  } catch (javax.ejb.FinderException fe) {
			   throw new HeroException(activityPerformerName+" does not exists");
		  }
		  try{
			 if (mUser.getEmail()!=null)	{
				   MailNotification mailNotif = new MailNotification();
				   //	logs trace
				   System.out.println ("sending mail to notify the user : " + activityPerformerName);
				   mailNotif.initMailService();									
				   mailNotif.sendMail(mUser.getEmail(),"Bonita Notification: activity assigned to you","Hello,\n\n For the Project: " + mProject.getName()+ ",\nthe activity for the node : " + nd.getName() + "\nis ready to be started by you.\n\nPlease connect to the Bonita manager application." );
			   }
		   } catch (Exception e){
			   throw new HeroException("Activity assign Notification fails" + e.getMessage());
		   }
				
	    } catch (HeroException he) {
		   he.printStackTrace();
		   throw new HeroException("Error in performerAssignment" + he.getMessage());
	    }		 
	}

	/**
	 * Method that consult new node state value and modify out edges (state 
	 * propagation) 
	 */

	private void modifyNodeEdges(int nodeState, Collection edges)
		throws HeroException, EngineException {
		int newEdgeState;
		Iterator i = edges.iterator();
		//	newEdgeState = hero.interfaces.Constants.edgeTransition[evaluateCondition(edge)][nodeState];
		while (i.hasNext()) {
			BnEdgeLocal edge = (BnEdgeLocal) i.next();
			boolean inNode = nodeExistInIteration(edge.getInBnNode().getName());
			boolean outNode = nodeExistInIteration(edge.getOutBnNode().getName());
			if ((inNode && outNode) || (!inNode && !outNode) ||(!inNode && outNode)) // ! StandBy edges (for nodes iteration)
			{
			  if (nodeState != hero.interfaces.Constants.Nd.TERMINATED) {
				newEdgeState =hero.interfaces.Constants.edgeTransition[Constants.CONDITION_TRUE][nodeState];
				mPSession.setEdgeState(edge, newEdgeState);
				edgeChanged(edge);
			  } else {
			  	  newEdgeState =hero.interfaces.Constants.edgeTransition[evaluateCondition(edge.getCondition(),edge.getInBnNode())][nodeState];
				  if (edge.getState() != newEdgeState) {
					mPSession.setEdgeState(edge, newEdgeState);
					edgeChanged(edge);
				  }
			  }
			}
		}
	}

	/**
	 * Method that find the existing nodes between lastNode and firstNode. 
	*/

	private Collection getWay(String lastNode, String firstNode) throws HeroException {
		String lN = lastNode;
		ArrayList nodes = new ArrayList();
		ArrayList disNodes = new ArrayList();
		boolean stop = false; int index=0;
		while (!stop)
		{
			Collection edges = mPSession.getNodeInEdges(lastNode);
			if (edges.size()==0 && !lastNode.equals(firstNode))
				disNodes.add(lastNode);
			if (lastNode.equals(firstNode))
				break;
			Iterator i = edges.iterator();
			while (i.hasNext())
			{
				String edgeName = (String)i.next();
				if (!nodes.contains(mPSession.getEdgeInNode(edgeName)))
					nodes.add(mPSession.getEdgeInNode(edgeName));
				
			}
			if (nodes.size()==0)
				nodes.add(firstNode);
			if (index<nodes.size())
				lastNode = (String)nodes.get(index);
			else
				stop=true;
			index++;
		} 
		if (nodes.contains(firstNode))
			nodes.remove(firstNode);
		for(index=0;index<disNodes.size();index++)
			nodes.remove(disNodes.get(index));

		int end = 0; boolean found = false;
		while (end<disNodes.size())
		{
		  boolean existNodes=true;
		  String nds = (String)disNodes.get(end);
		  String nodeOut=nds;
		  while (existNodes)
		  {
		    Collection eds = mPSession.getNodeOutEdges(nds);
		    Iterator iEds = eds.iterator();
		    while (iEds.hasNext())
		    {
		  	  String edgeO = (String)iEds.next();
			  nodeOut = mPSession.getEdgeOutNode(edgeO);
			  Collection edgesIn= mPSession.getNodeInEdges(nodeOut);
			  Iterator iEds2 = edgesIn.iterator();
			  while (iEds2.hasNext())
			  {
				String edgeI = (String)iEds2.next();
				String nodeI = mPSession.getEdgeInNode(edgeI);
			    if (nodes.contains(nodeI))
				  found = true;
			  }
			  if (!found)
				 nodes.remove(nodeOut);
			found=false;	
		    }
		  //found=false;	
		  //end++;
		  if (nodeOut.equals(firstNode) || nodeOut.equals(lN))
			existNodes=false;
		  nds=nodeOut;
		  }
		  end++;
		}
		return((Collection)nodes);
	}	
	
	/**
	 * Method that find the standBy edges of the iteration and active them. 
	*/

	private void activeStandByEdges(String lastNode, String firstNode) throws HeroException {
		ArrayList nodes = new ArrayList();
		boolean stop = false; int index=0;
		
		Collection activeEdges = mPSession.getNodeOutEdges(lastNode);
		Iterator j = activeEdges.iterator();
		while (j.hasNext())
		{
		  String aEdge = (String)j.next();
		  if (!nodes.contains(mPSession.getEdgeOutNode(aEdge)))
		  {
			 BnEdgeLocal ed = mProject.getBnEdge(aEdge);
			 int newEdgeState =hero.interfaces.Constants.edgeTransition[evaluateCondition(ed.getCondition(),ed.getInBnNode())][mPSession.getNodeState(lastNode)];
			 
		     mPSession.setEdgeState(ed, newEdgeState);
			 edgeChanged(ed);
    	  }
		}
		while (!stop)
		{
			Collection edges = mPSession.getNodeInEdges(lastNode);
			Iterator i = edges.iterator();
			while (i.hasNext())
			{
				String edgeName = (String)i.next();
				if (!nodes.contains(mPSession.getEdgeInNode(edgeName)))
					nodes.add(mPSession.getEdgeInNode(edgeName));
			}
			if (nodes.size()==0)
				nodes.add(firstNode);
			String actNode = (String)nodes.get(index);
			activeEdges = mPSession.getNodeOutEdges(actNode);
			j = activeEdges.iterator();
			while (j.hasNext())
			{
				String aEdge = (String)j.next();
				if (!nodes.contains(mPSession.getEdgeOutNode(aEdge)))
				{
					BnEdgeLocal ed = mProject.getBnEdge(aEdge);
					int newEdgeState =hero.interfaces.Constants.edgeTransition[evaluateCondition(ed.getCondition(), ed.getInBnNode())][mPSession.getNodeState(actNode)];

					mPSession.setEdgeState(ed, newEdgeState);
					edgeChanged(ed);
				}
			}
			lastNode = (String)nodes.get(index);
			if (lastNode.equals(firstNode))
			{
				stop=true;
				nodes.remove(lastNode);
			}
			index++;
		}
	}	
	
	/**
	 * Method that find the existing nodes between lastNode and firstNode) 
	*/

	private boolean nodeExistInIteration(String node) throws HeroException {
		Collection properties = mPSession.getProperties();
		Collection iterations = mPSession.getIterations();
		Iterator ite = iterations.iterator();
		while (ite.hasNext())
		{
			BnIterationLightValue iv = (BnIterationLightValue)ite.next(); 
			Collection way = this.getWay(iv.getFromNode(),iv.getToNode());
			if (node.equals(iv.getFromNode()) || node.equals(iv.getToNode()) || way.contains(node))
			  return true;
			
		}	
		return false;
	}
	

	/**
	 * Method that propagate the AgentEdges state
	 */

	private void modifyAgentEdges(Collection edges)
		throws HeroException, EngineException {
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnAgentEdgeLocal edge = (BnAgentEdgeLocal) i.next();
			mPSession.setAgentEdgeState(
				edge,
				hero.interfaces.Constants.Ed.ANTICIPATING);
			BnNodeLocal nd = edge.getOutBnNode();
			int operation = nd.getActivation().state(this.getNodeInEdges(nd));
			execute(nd.getName(), hero.interfaces.Constants.Nd.ANTACTIVE);
		}
	}

	private void edgeChanged(BnEdgeLocal edge)
		throws HeroException, EngineException {
		BnNodeLocal nd = edge.getOutBnNode();
		if (nd.getState() != hero.interfaces.Constants.Nd.DEAD) {
			int operation = nd.getActivation().state(this.getNodeInEdges(nd));
			execute(nd.getName(), operation);
		}
	}
	
	private Collection getNodeInEdges(BnNodeLocal nd ){
		Collection edgeValues = new ArrayList();
		Collection edges = nd.getInBnEdges();
		Iterator nodeEdges = edges.iterator();
		while (nodeEdges.hasNext())
			edgeValues.add(((BnEdgeLocal)nodeEdges.next()).getBnEdgeValue());
		return edgeValues;
    	    	
	}

	private boolean activityStarted(int state) throws HeroException {
		return (
			state == hero.interfaces.Constants.Nd.ANTICIPATING
				|| state == hero.interfaces.Constants.Nd.EXECUTING);
	}
	
	private void hookExecute(String nodeName, String eventName)
		throws HeroException, HeroHookException {
		Collection hooks;
		Collection interHooks;
		BnNodeLocal node;
		log.debug("hookExecute " + nodeName + " " + eventName);

		try {
			node = mProject.getBnNode(nodeName);
			hooks = node.getBnHooks();
			for (Iterator i = hooks.iterator(); i.hasNext();) {
				BnNodeHookLocal hk = (BnNodeHookLocal) i.next();
				if (hk.getEvent().equalsIgnoreCase(eventName)) {
					hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
					h.execute(this, eventName, node);
				}
			}

			interHooks = node.getBnInterHooks();
			for (Iterator i = interHooks.iterator(); i.hasNext();) {
				BnNodeInterHookLocal hk = (BnNodeInterHookLocal) i.next();
				if (hk.getEvent().equalsIgnoreCase(eventName)) {
					hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
					h.execute(this, eventName, node);
				}
			}

			hooks = mProject.getBnHooks();
			for (Iterator i = hooks.iterator(); i.hasNext();) {
				BnProjectHookLocal hk = (BnProjectHookLocal) i.next();
				if (hk.getEvent().equalsIgnoreCase(eventName)) {
					hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
					h.execute(this, eventName, node);
				}
			}

			interHooks = mProject.getBnInterHooks();
			for (Iterator i = interHooks.iterator(); i.hasNext();) {
				BnProjectInterHookLocal hk = (BnProjectInterHookLocal) i.next();
				if (hk.getEvent().equalsIgnoreCase(eventName)) {
					hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
					h.execute(this, eventName, node);
				}
			}
		}catch(HeroHookException hh){
		  //try{
			if (eventName.equals(hero.interfaces.Constants.Nd.AFTERSTART) || eventName.equals(hero.interfaces.Constants.Nd.BEFORETERMINATE)
					|| eventName.equals(hero.interfaces.Constants.Nd.ANTICIPATE) || eventName.equals(hero.interfaces.Constants.Nd.ONCANCEL)) {
				mContext.setRollbackOnly();
				throw new HeroHookException("Cannot execute hook "+hh.getMessage());
			}
			else 
				raiseError = true;
		  //}catch(Exception ut){throw new HeroHookException("An error was produced when rollback operations  "+ut.getMessage());}
		};
	}

	private boolean nodesTerminated(Collection nodes) {
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal node = (BnNodeLocal) i.next();
			int state = node.getState();
			if (state != hero.interfaces.Constants.Nd.TERMINATED
				&& state != hero.interfaces.Constants.Nd.EXPIRED
				&& state != hero.interfaces.Constants.Nd.DEAD)
				return false;
		}
		return true;
	}

	public int evaluateCondition(String condition, BnNodeLocal node) {
		try {
			if (condition != null) {
				condition = "import hero.interfaces.BnNodeLocal;\n"
						+ "boolean evalCondition (Object node) {\n\n\n"
						+ "return("
						+ condition
						+ ");"
						+ "}";
				Interpreter i = new Interpreter();

				// BnNode properties mapping

				Collection prop = node.getBnProperties();
				for (Iterator props = prop.iterator(); props.hasNext();) {
					BnNodePropertyLocal pl = (BnNodePropertyLocal) props.next();
					i.set(pl.getTheKey(), pl.getTheValue());
				}
				
				 //Project properties mapping
				 BnProjectLocal project = node.getBnProject();
				 Collection projectProp = project.getBnProperties();
				 for (Iterator props=projectProp.iterator();props.hasNext();) {
					 BnProjectPropertyLocal pP=(BnProjectPropertyLocal)props.next();
					 i.set(pP.getTheKey(),pP.getTheValue());
				 }
				i.set("node",node);
				i.eval(condition);
				i.eval("result=evalCondition(node)");
				if (((Boolean) i.get("result")).booleanValue() == true)
					return (Constants.CONDITION_TRUE);
				else
					return (Constants.CONDITION_FALSE);
			} else
				return (Constants.CONDITION_TRUE);
		} catch (Exception t) {
			t.printStackTrace();
			return (Constants.CONDITION_FALSE);
		}
	}
	
	private String logActivityProperties(String nodeName, String prelog) throws HeroException{
		Collection np = mPSession.getNodeProperties(nodeName);
		Iterator i = np.iterator();
		String thelog=prelog;
		if(np.size()!=0) {
			while (i.hasNext()){
				thelog= thelog + " key= '";
				BnNodePropertyValue property = (BnNodePropertyValue)i.next();
				thelog=thelog + property.getTheKey();
				thelog=thelog + "' value = '" + property.getTheValue() + "'";
			}
		}
		else
			thelog=prelog + " No property";
		return thelog;
	}
	private String logProjectProperties(String prelog) throws HeroException{
			Collection np = mPSession.getProperties();
			Iterator i = np.iterator();
			String thelog=prelog;
			if(np.size()!=0) {
				while (i.hasNext()){
					thelog= thelog + " key= '";
					BnProjectPropertyValue property = (BnProjectPropertyValue)i.next();
					thelog=thelog + property.getTheKey();
					thelog=thelog + "' value = '" + property.getTheValue() + "'";
				}
			}
			else
				thelog=prelog + " No property";
			return thelog;
		}


	/**
	 * init the Engine Session Bean
	 *
	 * @throws HeroException 
	 * @param projectName project name 
	 * @ejb:interface-method view-type="both"
	 **/
	public void initProject(String projectName) throws CreateException {
		log.debug("ejbCreate: project=" + projectName);
	}

	/**
	 *
	 * @throws CreateException 
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {
		try{
			JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
			jms=home.create();
     	}catch(Exception e){throw new CreateException(e.getMessage());}
	}

	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
	}

	public void ejbRemove() {
	}

	public void ejbActivate() {
	}

	public void ejbPassivate() {
}

}
