package hero.session;
/*
*
* ProjectSoapXMLBean.java - 
* Copyright (C) 2003 Ecoo Team
* valdes@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.interfaces.Constants;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnEdgeLocalHome;
import hero.interfaces.BnEdgeValue;
import hero.interfaces.BnNodeHookLocal;
import hero.interfaces.BnNodeHookValue;
import hero.interfaces.BnNodeInterHookLocal;
import hero.interfaces.BnNodeLightValue;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnAgentEdgeLocal;
import hero.interfaces.BnAgentEdgeLocalHome;
import hero.interfaces.BnAgentEdgeValue;
import hero.interfaces.BnAgentLocal;
import hero.interfaces.BnAgentLocalHome;
import hero.interfaces.BnAgentValue;
import hero.interfaces.BnProjectHookLocal;
import hero.interfaces.BnProjectHookValue;
import hero.interfaces.BnProjectInterHookLocal;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectPropertyLocal;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.BnProjectValue;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.BnRoleLocalHome;
import hero.interfaces.BnRoleValue;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserUtil;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.interfaces.JMSServicesSessionUtil;
import hero.interfaces.JMSServicesSessionLocalHome;
import hero.interfaces.JMSServicesSessionLocal;


import hero.util.NodeHooks;
import hero.util.NodeInEdges;
import hero.util.NodeOutEdges;
import hero.util.NodeProperties;
import hero.util.ProjectHooks;
import hero.util.ProjectNodes;
import hero.util.ProjectProperties;
import hero.util.ProjectRoles;
import hero.util.ProjectUsers;
import java.io.StringWriter;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import javax.naming.Context;

import org.apache.log4j.Category;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.ValidationException;
import hero.util.StrutsNodeValue;


/**
 * ProjectSessionXML API based on ProjectSession API but oriented to web services 
 * calls from a non-Java language. Get methods of this API returns
 * XML.
 *
 * @ejb:bean name="ProjectSoapXML"
 *           display-name="ProjectSoapXML Bean"
 *           type="Stateful"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/ProjectSoapXML"
 *
 * @ejb:ejb-ref	ejb-name="ProjectSoapXML"
 *              	view-type="remote"
 *              	ref-name="ProjectSoapXML"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 *
 * @ejb:transaction type="Supports"
 * @ejb.permission role-name="Admin,user"
 * 
 * @jonas.bean
 *      ejb-name="ProjectSoapXML"
 *      jndi-name="ejb/hero/ProjectSoapXML"
 * 
 * @copyright INRIA
 * @author Miguel Valdes
 **/

public class ProjectSoapXMLBean implements SessionBean, EventConstants {

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Category log = Category.getInstance(ProjectSoapXMLBean.class);

	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	private BnProjectLocalHome pHome;
	private BnProjectLocal mProject;
	private JMSServicesSessionLocal jms;
	private Collection subProjects = new ArrayList();
	private ArrayList visited = new ArrayList();

	// -------------------------------------------------------------------------
	// XML Get Methods (Castor Data Binding)
	// -------------------------------------------------------------------------

	/**
	 * Get ProjectName
	 * @return String 
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getName() throws HeroException {
		log.debug("getName:");
		return mProject.getName();
	}

	/**
	 * Get project information
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getProjectDetails() throws HeroException {

		try {
			//BnProjectLightValue pl = mProject.getBnProjectLightValue();
			BnProjectValue pl = mProject.getBnProjectValue();
			StringWriter sw = new StringWriter();
			Marshaller.marshal(pl, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get node information
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getNodeDetails(String node) throws HeroException {

		try {
			BnNodeLocal nd = mProject.getBnNode(node);
			BnNodeLightValue nl = nd.getBnNodeLightValue();

			StringWriter sw = new StringWriter();			
			Marshaller.marshal(nl, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	
	/**
	 * Get the node Light Value
	 *
	 * @param name the name of the Node
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeLightValue(String name)
		throws HeroException {
		log.debug("getBnNode: name=" + name);
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		BnNodeLightValue nv;
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mNode = nHome.findByName(name, mProject.getId());
			nv = mNode.getBnNodeLightValue();
			StringWriter sw = new StringWriter();			
			Marshaller.marshal(nv, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		} catch (FinderException fe) {
			throw new HeroException("In BnNode does not exist");
		}
	}

	/**
	 * Get node deadline
	 * @return String 
	 *
	 * @param nodeName The name of the node
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeDeadline(String nodeName) throws HeroException {
		log.debug("getBnNodeDeadline: nodeName=" + nodeName);
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		if (nd.getDeadline() != null)
			return (nd.getDeadline().toString());
		else
			return (null);
	}
	
	/**
	 * Get the node description
	 *
	 * @param name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeDescription(String name) throws HeroException {
		log.debug("setBnNodeDescription: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		return (nd.getDescription());
	}

	/**
	 * Get node role information
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getNodeRole(String node) throws HeroException {

		try {
			BnNodeLocal nd = mProject.getBnNode(node);
			BnRoleValue rl = ((BnRoleLocal) nd.getBnRole()).getBnRoleValue();

			StringWriter sw = new StringWriter();
			Marshaller.marshal(rl, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	
	/**
	 * Get node role name
	 * @return String
	 *
	 * @param nodeName The name of the node
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeRoleName(String nodeName) throws HeroException {
		log.debug("getBnNodeRoleName: nodeName=" + nodeName);
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		return (((BnRoleLocal) nd.getBnRole()).getName());
	}
	
	/**
	 * Get the Agent Value
	 *
	 * @param name the name of the BnAgent
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getAgentValue(String name) throws HeroException {
		log.debug("getBnAgent: name=" + name);
		BnAgentLocalHome nHome;
		BnAgentLocal mAgent;
		BnAgentValue av;
		try {
			nHome = hero.interfaces.BnAgentUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mAgent = nHome.findByName(name, mProject.getId());
			av = mAgent.getBnAgentValue();
			StringWriter sw = new StringWriter();			
			Marshaller.marshal(av, sw);
			return (sw.toString());
		} catch (FinderException fe) {
			throw new HeroException("In Agent does not exist");
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	
	/**
	 * Get the AgentEdge Value
	 *
	 * @param name the name of the AgentEdge
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getAgentEdgeValue(String name)
		throws HeroException {
		log.debug("getBnAgentEdge: name=" + name);
		BnAgentEdgeLocalHome nHome;
		BnAgentEdgeLocal mAgentEdge;
		BnAgentEdgeValue av;
		try {
			nHome = hero.interfaces.BnAgentEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mAgentEdge = nHome.findByName(name, mProject.getId());
			av = mAgentEdge.getBnAgentEdgeValue();
			StringWriter sw = new StringWriter();			
			Marshaller.marshal(av, sw);
			return (sw.toString());
		} catch (FinderException fe) {
			throw new HeroException("In BnAgentEdge does not exist");
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}


	/**
	 * Get node executor information
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getNodeExecutor(String node) throws HeroException {

		try {
			BnNodeLocal nd = mProject.getBnNode(node);
			BnUserLightValue ul = ((BnUserLocal) nd.getExecutor()).getBnUserLightValue();

			StringWriter sw = new StringWriter();			
			Marshaller.marshal(ul, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get the state of the node
	 *
	 * @param name of the node
	 * @return The state of the node
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public int getNodeState(String name) throws HeroException {
		log.debug("setBnNodeState: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		return (nd.getState());
	}

	/**
	 * Get the type of the node
	 *
	 * @param name of the node
	 * @return The type of the node
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public int getNodeType(String name) throws HeroException {
		log.debug("setBnNodeType: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		return (nd.getType());
	}
	
	/**
	 * Get the node Value
	 *
	 * @param name the name of the BnNode
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeValue(String name) throws HeroException {
		log.debug("getBnNode: name=" + name);
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		BnNodeValue nv;
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mNode = nHome.findByName(name, mProject.getId());
			nv = mNode.getBnNodeValue();
			StringWriter sw = new StringWriter();			
			Marshaller.marshal(nv, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		} catch (FinderException fe) {
			throw new HeroException("This Node does not exist");
		}
	}

	/**
	 * Get all nodes of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getNodes() throws HeroException {

		ProjectNodes nb = new ProjectNodes();
		Collection nodes = new ArrayList();
		ArrayList ns = new ArrayList();
		StringWriter sw = new StringWriter();

		nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal n = (BnNodeLocal) i.next();
			StrutsNodeValue snv = new StrutsNodeValue();
			snv.setProjectName(mProject.getName());
			snv.setName(n.getName());
			snv.setCreator(n.getCreator().getName());
			snv.setRole(n.getBnRole().getName());
			snv.setState(Constants.Nd.nodeStateName[n.getState()]);
			snv.setType(Constants.Nd.nodeTypeName[n.getType()]);
			if (n.getDescription() != null)
				snv.setDescription(n.getDescription());
			if (n.getExecutor() != null)
				snv.setExecutor(n.getExecutor().getName());
			snv.setAnticipable(n.getAnticipable());
			if (n.getDeadline() != null)
				snv.setDeadline(n.getDeadline().toString());
						
			ns.add(snv);			
		}
		nb.setNodes(ns.toArray());

		try {
			Marshaller.marshal(nb, sw);
		} catch (MarshalException e) {
			e.printStackTrace();
		} catch (ValidationException e) {
			e.printStackTrace();
		}
		return (sw.toString());
	}

	/**
	 * Get all node names of the project
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getNodesNames() throws HeroException {
		log.debug("getBnNodesNames: ");

		//	Vector names = new Vector();
		ArrayList names = new ArrayList();
		Collection nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal n = (BnNodeLocal) i.next();
			names.add(n.getName());
		}
		return ((Collection) names);
	}

	/**
	 * Get all users of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getUsers() throws HeroException {

		ProjectUsers ub = new ProjectUsers();
		Collection us = new ArrayList();

		try {
			Collection users = mProject.getBnUsers();
			Iterator i = users.iterator();
			while (i.hasNext()) {
				BnUserLocal u = (BnUserLocal) i.next();
				BnUserLightValue ul = u.getBnUserLightValue();
				us.add(ul);
			}
			ub.setUser(us);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(ub, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	
	/**
	 * Get user roles in project names
	 * @return Collection 
	 *
	 * @param userName The name of the user
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getUserRolesInProjectNames(String userName)
		throws HeroException {
		log.debug("getUserRolesInProjectNames:");
		BnUserLocalHome uHome;
		BnUserLocal user;
		ArrayList result = new ArrayList();
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {
			throw new HeroException("FinderException");
		}

		Collection userRoles = user.getBnRoles();
		Collection projectRoles = mProject.getBnRoles();
		for (Iterator i = userRoles.iterator(); i.hasNext();) {
			BnRoleLocal ur = (BnRoleLocal) i.next();
			for (Iterator j = projectRoles.iterator(); j.hasNext();) {
				BnRoleLocal up = (BnRoleLocal) j.next();
				if (ur.getId() == up.getId())
					result.add(ur.getBnRoleValue().getName());
			}
		}
		return result;
	}

	/**
	 * Get all roles of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getRoles() throws HeroException {

		ProjectRoles rb = new ProjectRoles();
		Collection rs = new ArrayList();

		try {
			Collection roles = mProject.getBnRoles();
			Iterator i = roles.iterator();
			while (i.hasNext()) {
				BnRoleLocal r = (BnRoleLocal) i.next();
				BnRoleValue rl = r.getBnRoleValue();
				rs.add(rl);
			}
			rb.setRole(rs);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(rb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get all roles names of the project
	 * @return Collection 
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getRolesNames() throws HeroException {
		log.debug("getBnRolesNames:");
		Collection roles = mProject.getBnRoles();
		ArrayList result = new ArrayList();
		for (Iterator i = roles.iterator(); i.hasNext();) {
			result.add(((BnRoleLocal) i.next()).getName());
		}
		return result;
	}

	/**
	 * Get user roles for this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getUserRoles(String userName) throws HeroException {

		log.debug("getUserRoles: userName= "+userName);
		BnUserLocalHome uHome;
		BnUserLocal user;
		Collection result = new ArrayList();
		ProjectRoles rb = new ProjectRoles();
		try {
			uHome = (BnUserLocalHome) BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {
			throw new HeroException("FinderException");
		}
		Collection userRoles = user.getBnRoles();
		Collection projectRoles = mProject.getBnRoles();
		for (Iterator i = userRoles.iterator(); i.hasNext();) {
			BnRoleLocal ur = (BnRoleLocal) i.next();
			for (Iterator j = projectRoles.iterator(); j.hasNext();) {
				BnRoleLocal up = (BnRoleLocal) j.next();
				if (ur.getId() == up.getId())
					result.add(ur.getBnRoleValue());
			}
		}
		try {
			rb.setRole(result);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(rb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	
	/**
	 * Get user roles in project
	 * @return Collection 
	 *
	 * @param userName The name of the user
	 *
	 * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
	 *
	 **/
	public String getUserRolesInProject(String userName)
		throws HeroException {
		log.debug("getUserRolesInProject: userName= "+userName);
		BnUserLocalHome uHome;
		BnUserLocal user;
		ArrayList result = new ArrayList();
		ProjectRoles rb = new ProjectRoles();
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {
			throw new HeroException("FinderException");
		}
		Collection userRoles = user.getBnRoles();
		Collection projectRoles = mProject.getBnRoles();
		for (Iterator i = userRoles.iterator(); i.hasNext();) {
			BnRoleLocal ur = (BnRoleLocal) i.next();
			for (Iterator j = projectRoles.iterator(); j.hasNext();) {
				BnRoleLocal up = (BnRoleLocal) j.next();
				if (ur.getId() == up.getId())
					result.add(ur.getBnRoleValue());
			}
		}
		try {
		  rb.setRole(result);
		  StringWriter sw = new StringWriter();
		  Marshaller.marshal(rb, sw);
		  return (sw.toString());
		} catch (MarshalException ex1) {
		  throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
		  throw new HeroException(ex2.getMessage());
		}
	}


	/**
	 * Get the properties of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getProperties() throws HeroException {

		ProjectProperties pb = new ProjectProperties();
		Collection ps = new ArrayList();

		try {
			Collection properties = mProject.getBnProperties();
			Iterator i = properties.iterator();
			while (i.hasNext()) {
				BnProjectPropertyLocal pp = (BnProjectPropertyLocal) i.next();
				BnProjectPropertyValue ppv = pp.getBnProjectPropertyValue();
				ps.add(ppv);
			}
			pb.setProperty(ps);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(pb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get the hooks of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getHooks() throws HeroException {

		ProjectHooks hb = new ProjectHooks();
		Collection hs = new ArrayList();

		try {
			Collection hooks = mProject.getBnHooks();
			Iterator i = hooks.iterator();
			while (i.hasNext()) {
				BnProjectHookLocal ph = (BnProjectHookLocal) i.next();
				BnProjectHookValue phv = ph.getBnProjectHookValue();
				hs.add(phv);
			}
			hb.setHook(hs);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(hb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	
	/**
	 * Get Interactive hooks of the project
	 *
	 * @return A BnProjectHookValue collection of all project inter hooks
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getInterHooks() throws HeroException {
	 try{
		Collection hk = new ArrayList();
		ProjectHooks hb = new ProjectHooks();
		BnProjectInterHookLocal pl;
		Collection pls = mProject.getBnInterHooks();
		
		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnProjectInterHookLocal) i.next();
			hk.add(pl.getBnProjectInterHookValue());
		}
		hb.setHook(hk);
		StringWriter sw = new StringWriter();
		Marshaller.marshal(hb, sw);
		return (sw.toString());
	  } catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
	  } catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
	  }
	}
	
	/**
	 * Get Interactive Node hooks of the project
	 *
	 * @param nodeName the name of the node
	 * @return A NodeHookValue collection of all node hooks (XML)
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeInterHooks(String nodeName) throws HeroException {
	 try{
		Collection hk = new ArrayList();
		BnNodeInterHookLocal pl;
		ProjectHooks hb = new ProjectHooks();

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		Collection pls = nd.getBnInterHooks();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnNodeInterHookLocal) i.next();
			hk.add(pl.getBnNodeInterHookValue());
		}
		hb.setHook(hk);
		StringWriter sw = new StringWriter();
		Marshaller.marshal(hb, sw);
		return (sw.toString());
	  } catch (MarshalException ex1) {
		throw new HeroException(ex1.getMessage());
	  } catch (ValidationException ex2) {
		throw new HeroException(ex2.getMessage());
	  }
	}
	
	/**
	 * Get the node hook value. Only for interactives hooks
	 *
	 * @param name of the node
	 * @param hook of the node
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeInterHookValue(String node, String hook)
		throws HeroException {
		String value = null;
		log.debug("getBnNodeInterHookValue: node=" + node + " hook=" + hook);
		BnNodeLocal nd = mProject.getBnNode(node);
		Collection hooks = nd.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnNodeInterHookLocal hk = (BnNodeInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				value = hk.getScript();
			}
		}
		return value;
	}
	
	/**
	 * Get Node inter hook of the project
	 *
	 * @param nodeName the name of the node
	 * @param interHook name
	 * @return A BnNodeInterHookValue 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeInterHook(String nodeName, String interHook) throws HeroException {
		log.debug("getNodeInterHook: node=" + nodeName + " key=" + interHook);
		try{
		  BnNodeInterHookLocal hl;
		  BnNodeLocal nd = mProject.getBnNode(nodeName);
		  Collection hls = nd.getBnInterHooks();
	
		  for (Iterator i = hls.iterator(); i.hasNext();) {
			hl = (BnNodeInterHookLocal) i.next();
			if (hl.getName().equals(interHook)) {
			   StringWriter sw = new StringWriter();
			   Marshaller.marshal(hl.getBnNodeInterHookValue(), sw);
			   return (sw.toString());
			}
		  }
		 } catch (MarshalException ex1) {
			  throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
		      throw new HeroException(ex2.getMessage());
		} 
		return null;
	}
	
	/**
	 * Return all registered users
	 * @return A string collection of all user names
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
		public Collection getAllUsers() throws HeroException {
			log.debug("getAllUsers:");
			BnUserLocalHome uHome;
			Collection users;
			try {
				uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
			} catch (javax.naming.NamingException be) {
				throw new HeroException(be.getMessage());
			}
			try {
				users = uHome.findAll();
			} catch (FinderException f) {
				throw new HeroException("FinderException in findByAll users");
			}
			ArrayList result = new ArrayList();
			for (Iterator i = users.iterator(); i.hasNext();) {
				result.add(((BnUserLocal) i.next()).getName());
			}
			return result;
		}

	/**
	 * Get the node properties of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getNodeProperties(String nodeName) throws HeroException {

		NodeProperties nb = new NodeProperties();
		Collection ns = new ArrayList();

		try {
			BnNodeLocal nl = mProject.getBnNode(nodeName);
			Collection properties = nl.getBnProperties();
			Iterator i = properties.iterator();
			while (i.hasNext()) {
				BnNodePropertyLocal np = (BnNodePropertyLocal) i.next();
				BnNodePropertyValue npv = np.getBnNodePropertyValue();
				ns.add(npv);
			}
			nb.setProjectName(mProject.getName());
			nb.setProperty(ns);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(nb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	
	/**
	 * Get Node property of the project
	 *
	 * @param nodeName the name of the node
	 * @param key Property key
	 * @return A BnNodePropertyValue 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeProperty(String nodeName, String key) throws HeroException {
	  log.debug("getNodeProperty: node=" + nodeName + " key=" + key);
	  try{
		BnNodePropertyLocal pl;
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		Collection pls = nd.getBnProperties();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnNodePropertyLocal) i.next();
			if (pl.getTheKey().equals(key)) {
			   StringWriter sw = new StringWriter();
			   Marshaller.marshal(pl.getBnNodePropertyValue(), sw);
			   return (sw.toString());
			}
		}
	   } catch (MarshalException ex1) {
	     throw new HeroException(ex1.getMessage());
	   } catch (ValidationException ex2) {
	     throw new HeroException(ex2.getMessage());
	   }
	 return (null);
	}

	/**
	 * Get the node hooks of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getNodeHooks(String nodeName) throws HeroException {
		NodeHooks nb = new NodeHooks();
		Collection hs = new ArrayList();

		try {
			BnNodeLocal nl = mProject.getBnNode(nodeName);
			Collection hooks = nl.getBnHooks();
			Iterator i = hooks.iterator();
			while (i.hasNext()) {
				BnNodeHookLocal nh = (BnNodeHookLocal) i.next();
				BnNodeHookValue nhv = nh.getBnNodeHookValue();
				hs.add(nhv);
			}
			nb.setProjectName(mProject.getName());
			nb.setHook(hs);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(nb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get creator of the project
	 * @return String 
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getCreator() throws HeroException {
		log.debug("getCreator:");
		return mProject.getCreator();
	}

	/**
	* get all in edges of the node
	* @return String
	*
	* @param name the name ot the node
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Supports"
	*
	**/
	public String getNodeInEdges(String name) throws HeroException {

		NodeInEdges eb = new NodeInEdges();
		Collection in = new ArrayList();

		try {
			BnNodeLocal nd = mProject.getBnNode(name);

			Collection edgesIn = nd.getInBnEdges();
			Iterator i = edgesIn.iterator();
			while (i.hasNext())
				in.add(((BnEdgeLocal) i.next()).getBnEdgeValue());

			eb.setProjectName(mProject.getName());
			eb.setInEdge(in);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(eb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get the edge Condition
	 *
	 * @param edge name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getEdgeCondition(String edge)
		throws HeroException {
		log.debug("getEdgeCondition: name=" + edge);
		BnEdgeLocal e = mProject.getBnEdge(edge);
		return(e.getCondition());
	}
	
	/**
	 * Get edge in node
	 * @return String 
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getEdgeInNode(String edgeName) throws HeroException {
		log.debug("getBnEdgeInNode: edgeName" + edgeName);
		BnEdgeLocal e = mProject.getBnEdge(edgeName);
		return (((BnNodeLocal) e.getInBnNode()).getName());
	}

	/**
	 * Get edge out node
	 * @return String 
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getEdgeOutNode(String edgeName) throws HeroException {
		log.debug("getBnEdgeOutNode: edgeName" + edgeName);
		BnEdgeLocal e = mProject.getBnEdge(edgeName);
		return (((BnNodeLocal) e.getOutBnNode()).getName());
	}
	
	/**
	 * Get all edges of the project
	 * @return Collection 
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getEdgesNames() throws HeroException {
		log.debug("getBnEdges:");
		//	Vector names = new Vector();
		ArrayList names = new ArrayList();
		Collection edges = mProject.getBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal n = (BnEdgeLocal) i.next();
			names.add(n.getName());
		}
		return ((Collection) names);
	}

	/**
	* get all out edges of the node
	* @return Collection 
	*
	* @param name the name ot the node
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Supports"
	*
	**/
	public String getNodeOutEdges(String name) throws HeroException {

		NodeOutEdges eb = new NodeOutEdges();
		Collection out = new ArrayList();

		try {
			BnNodeLocal nd = mProject.getBnNode(name);

			Collection edgesOut = nd.getOutBnEdges();
			Iterator i = edgesOut.iterator();
			while (i.hasNext())
				out.add(((BnEdgeLocal) i.next()).getBnEdgeValue());

			eb.setProjectName(mProject.getName());
			eb.setOutEdge(out);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(eb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	
	/**
	 * Get the Edge Value
	 *
	 * @param name the name of the Edge
	 *
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getEdgeValue(String name) throws HeroException {
		log.debug("getBnEdge: name=" + name);
		BnEdgeLocalHome eHome;
		BnEdgeLocal mEdge;
		BnEdgeValue ev;
		try {
			eHome = hero.interfaces.BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mEdge = eHome.findByName(name, mProject.getId());
			ev = mEdge.getBnEdgeValue();
			StringWriter sw = new StringWriter();
			Marshaller.marshal(ev, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		} catch (FinderException fe) {
			throw new HeroException("BnEdge does not exist");
		}
	}
	
	/**
	 * Get the node in traditional mode
	 *
	 * @param name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public boolean getNodeAnticipable(String name) throws HeroException {
		log.debug("getBnNodeTraditional: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		return (nd.getAnticipable());
	}
	
	/**
	  * Get the inter hook value script
	  *
	  * @param script of the inter hook
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Supports"
	  *
	  **/
		public String getInterHookValue(String hook) throws HeroException {
		  String value= null;
		  log.debug("setBnNodeInterHookValue:  hook=" + hook);
		  Collection hooks = mProject.getBnInterHooks();
		  for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			 BnProjectInterHookLocal hk = (BnProjectInterHookLocal) ih.next();
			 if (hk.getName().equals(hook)) {
				value=hk.getScript();
			 }
		  }
		  return value;
	   }

	/**
	 * Get the project name of this instance
	 *
	 * @param instanceName the name of the instance
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getProjectNameOfInstance(String instanceName) throws HeroException {
		log.debug("getProjectNameOfInstance: instanceName=" + instanceName);
		if (instanceName.matches(".*"+"_instance"+".*")){
			int i = instanceName.indexOf("_instance");
			return(instanceName.substring(0,i));
		}
		else
		throw new HeroException(instanceName+" is not a project instance...");
	}
	
	/**
	 * Init the Project Session Bean. You have to call this method after "create" call. After that
	 * all methods of ProjectSession API are available.
	 *
	 * @param projectName project name 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 **/
	public void initProject(String projectName) throws HeroException {
		Context lContext;
		BnUserLocalHome uhome = null;
		BnRoleLocalHome rhome = null;
		log.debug("initProject: project=" + projectName);
		  try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			this.mProject = pHome.findByName(projectName);
		  } catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		  } catch (javax.ejb.FinderException fe) {
				throw new HeroException(fe.getMessage());
		   }
	}

	/**
	 * Creates the Project Session Bean. This method is the first one to invoke in order to
	 * use ProjectSession API. If the user is not authorized this method throws an exception.
	 *
	 * @throws CreateException 
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/
	public void ejbPostCreate(String oldProject, String newProject)
		throws CreateException, RemoteException {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/
	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
		try{
		  JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
		  this.jms=home.create();
		}catch(Exception e){throw new EJBException(e.getMessage());}
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/

	public void ejbRemove() {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/

	public void ejbActivate() {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/

	public void ejbPassivate() {
	}
	
}

