package hero.session;
/*
* 01/02/2004 - 16:36:12
*
* UserSessionEJB.java - 
* Copyright (C) 2004 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodeUtil;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLightValue;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserPropertyLocal;
import hero.interfaces.BnUserPropertyLocalHome;
import hero.interfaces.BnUserPropertyUtil;
import hero.interfaces.BnUserValue;
import hero.interfaces.EngineLocal;
import hero.interfaces.EngineLocalHome;
import hero.interfaces.InvalidValueException;
import hero.interfaces.ProjectSessionLocal;
import hero.interfaces.ProjectSessionLocalHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.JMSServicesSessionUtil;
import hero.interfaces.JMSServicesSessionLocalHome;
import hero.interfaces.JMSServicesSessionLocal;
import hero.util.EngineException;
import hero.util.HeroException;
import hero.util.EventConstants;
import hero.interfaces.Constants;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

/**
 *
 * The User Session Bean, is an stateful session bean that provides the user API to get information on
 * Todo list and started activities and to produce events on activities (start, terminate, cancel).<br>
 * This Session Bean is based on Engine Session Bean: a recursive implementation that manage the previous execution operations
 * and propagates the activity state changes to the activities that are connected to this one.
 *<br><br>
 * The User Session Bean API provides information about user projects and activites (project list,
 * todo list and activity list) and also useful information about project instances or user
 * preferences. With this API users can performs his task/activities by using start, terminate and
 * cancel methods and also terminates workflow processes.
 * <br><br>
 * <strong>The following lines shows a sample code to use this API in your application:<br><br></strong>
 * <br>
 * First of all you have to import the User Session files:<br>
 * <br>
 * import hero.interfaces.UserSessionLocalHome;<br>
 * import hero.interfaces.UserSessionLocal;<br>
 * import hero.interfaces.UserSessionHome;<br>
 * import hero.interfaces.UserSession;<br>
 * import hero.interfaces.UserSessionUtil;<br>
 * <br>
 * Now, it is time to create the User Session instance:<br>  
 * <br>
 * Like this if you want to use local interfaces:<br><br>
 * UserSessionLocalHome userh = (UserSessionLocalHome)hero.interfaces.UserSessionUtil.getLocalHome();<br>
 * UserSessionLocal usersession = userh.create();<br>
 * <br>
 * or like this if you use remote interfaces:<br>  
 * <br>
 * UserSessionHome userh = (UserSessionHome)hero.interfaces.UserSessionUtil.getHome();<br>
 * UserSession usersession = userh.create();<br>
 * <br>
 * <br>
 * after that, you can init the User Session API for this user by using the init method:<br><br>
 * usersession.init();<br>
 * <br>
 * Now you can call all User Sessions methods...  
 * 
 *
 * @ejb:bean name="UserSession"
 *           display-name="UserSession Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/UserSession"
 *           local-jndi-name="ejb/hero/UserSession_L"
 *
 * @ejb:ejb-ref ejb-name="BnUser"
 *              ref-name="myhero/BnUser"
 * @ejb.permission role-name="Admin,user"
 *
 * @jonas.bean
 *      ejb-name="UserSession"
 *      jndi-name="ejb/hero/UserSession"
 *
 * 
 * 
 * @copyright INRIA
 * @author Miguel Valdes
 **/

public class UserSessionBean implements SessionBean, EventConstants {

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Logger log = Logger.getLogger(UserSessionBean.class);

	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------

	/**
	 * Get user project list. Workflow processes associated to this user
	 * @return a Collection of BnProjectLightValue objects - the projects list of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProjectList() throws HeroException {
			Collection pjts = new ArrayList();
			Collection pva;
			try{
				BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
				log.info("start by " + mContext.getCallerPrincipal().getName());
				//Collection projects = null;
				//projects = mUser.getBnProjects();
				BnUserValue uv = mUser.getBnUserValue();
				pva = new ArrayList(Arrays.asList(uv.getBnProjects()));
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			} catch (javax.ejb.FinderException fe) {
				log.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
				throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
			}
			log.info("End");
			return pva;
		}
	/**
	 * Get project list names for this user
	 * @return a String Collection - the projects list names of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProjectListNames() throws HeroException {
			log.info("start by " + mContext.getCallerPrincipal().getName());
			//Collection projects = null;
			Collection projectsNames;
			try{
			  BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			  BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
			  //projects = mUser.getBnProjects();
			  BnUserValue uv = mUser.getBnUserValue();
			  Collection pva = new ArrayList(Arrays.asList(uv.getBnProjects()));
			  projectsNames = new ArrayList();
			  Iterator i = pva.iterator();
			  while (i.hasNext())
				 projectsNames.add(((hero.interfaces.BnProjectLightValue) i.next()).getName());
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
			  throw new HeroException(ne.getMessage());
			} catch (javax.ejb.FinderException fe) {
				log.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
			  throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
			}
			log.info("End");
			return projectsNames;
		}

	/**
	 * Get user instances list. This method is equivalent to getProjectList but
	 * it only returns the current instances of the user.
	 * @return a Collection of BnProjectLightValue objects - the projects list of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getInstancesList() throws HeroException {
		log.info("start by " + mContext.getCallerPrincipal().getName());
		Collection instances = new ArrayList();
		Collection pva = this.getProjectList();
		Iterator ps = pva.iterator();
		
		while (ps.hasNext())
		{
			BnProjectLightValue pro = (BnProjectLightValue)ps.next();
			if (pro.getName().matches(".*_instance.*"))
				instances.add(pro);
		}
		log.info("End");
		return instances;
	}

	/**
	 * Get instances list names for this user. This method is equivalent to getProjectListNames but
	 * it only returns the current instances of the user.
	 * @return a String Collection - the projects list names of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getInstancesListNames() throws HeroException {
		log.info("start by " + mContext.getCallerPrincipal().getName());
		
		Collection instances = new ArrayList();
		Collection pva = this.getProjectList();
		Iterator ps = pva.iterator();
		while (ps.hasNext())
		{
			BnProjectLightValue pro = (BnProjectLightValue)ps.next();
			if (pro.getName().matches(".*_instance.*"))
			  instances.add(pro.getName());
		}
		log.info("End");
		return instances;
	}
	
	/**
	 * Delete a Workflow project
	 * @param projectName the name of the project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void removeProject(String projectName) throws HeroException {
			log.info("parameter: projectName=" + projectName + " start by " + mContext.getCallerPrincipal().getName());
			BnProjectLocalHome pHome;
			JMSServicesSessionLocal jms;
			try{
			  JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
			  jms=home.create();
			}catch(Exception e){
				log.error(e.getMessage());
				throw new EngineException(e.getMessage());
				}
			try {
				pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			} catch (javax.naming.NamingException be) {
				log.error(be.getMessage());
				throw new HeroException(be.getMessage());
			}
			try {
				BnProjectLocal pl = pHome.findByName(projectName);
				String userName = mContext.getCallerPrincipal().getName();
				if (userName.equals(pl.getCreator()))  // Only project creator must delete the project
				{
				  Collection nodes = pl.getBnNodes();
				  Iterator nds = nodes.iterator();
				  while (nds.hasNext())            // Verifies if the project has any subProcesses
				  {
					BnNodeLocal nd = (BnNodeLocal)nds.next();
					if (nd.getType()==Constants.Nd.SUB_PROCESS_NODE)
					   throw new HeroException("Error in delete project: You have to delete subProcesses first !!");
				  }
				  if (pl.getParent().equals(pl.getName()))
				  {
					pl.remove();
					jms.sendProjectEvent(DELETEPROJECT, projectName);
				  }
				  else
					throw new HeroException("Error in delete project: You have to delete subProcess activity of this project !!");
				}
				else
					throw new HeroException("Error in delete project: only the project creator can removed it !!");
			} catch (javax.ejb.FinderException fe) {
				log.error("User does not exist " + fe.getMessage());
				throw new HeroException("User does not exist");
			} catch (RemoveException re) {
				log.error("Error in delete project " + re.getMessage());
				throw new HeroException("Error in delete project");
		}
		log.info("End");
		}
		
	/**
	* Obtain all user activities from specific project (ready and anticipable state)
	* @param projectName the name of the project
	* @return a String Collection - the list of todo activities of the user for a project
	* @throws HeroException
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/
	public Collection getToDoList(String projectName) throws HeroException {
			log.info("parameter: projectName=" + projectName + " start by " + mContext.getCallerPrincipal().getName());
			BnProjectLocalHome projhome;
			BnProjectLocal project;
			BnNodeLocalHome nodehome;
			Collection ready;
			try {
				projhome = BnProjectUtil.getLocalHome();
				nodehome = BnNodeUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			}
			try {
				project = projhome.findByName(projectName);
			} catch (FinderException fe) {
				log.error("BnProject " + projectName + " does not exists " + fe.getMessage());
				throw new HeroException("BnProject " + projectName + " does not exists");
			}
			try {
				ready = nodehome.findUserReady(project.getId(), mContext.getCallerPrincipal().getName());
			} catch (FinderException fe) {
				log.error(fe.getMessage());
				ready = new ArrayList();
			}
			ArrayList result = new ArrayList();
			for (Iterator i = ready.iterator(); i.hasNext();) {
				BnNodeLocal nd = (BnNodeLocal) i.next();
				log.info("NODE NAME=" + nd.getName());
				result.add(nd.getName());
			}
			log.info("End");
			return result;
		}

	/**
	 * Obtain all user activities from specific project (executing and anticipating state)
	 *
	 * @param projectName the name of the project
	 * @return a String Collectio - the list of active activities of the user for a project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getActivityList(String projectName)
			throws HeroException {
			BnProjectLocalHome projhome;
			BnProjectLocal project;
			BnNodeLocalHome nodehome;
			Collection executing;
			log.info("parameter: -begin projectName=" + projectName + " start by " + mContext.getCallerPrincipal().getName());
			try {
				projhome = BnProjectUtil.getLocalHome();
				nodehome = BnNodeUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			}
			try {
				project = projhome.findByName(projectName);
			} catch (FinderException fe) {
				log.error("BnProject " + projectName + " does not exists " + fe.getMessage());
				throw new HeroException("BnProject " + projectName + " does not exists");
			}
			try {
				executing =
					nodehome.findUserExecuting(project.getId(), mContext.getCallerPrincipal().getName());
			} catch (FinderException fe) {
				log.error(fe.getMessage());
				executing = new ArrayList();
			}
			ArrayList result = new ArrayList();
			for (Iterator i = executing.iterator(); i.hasNext();) {
				BnNodeLocal nd = (BnNodeLocal) i.next();
				result.add(nd.getName());
			}
			log.info("End");
			return result;
		}	
		
	/**
	* Obtain todo user activities for all instances (ready and anticipable state)
	* @return a BnNodeValue Collection - the list of todo activities of the user for all instances
	* @throws HeroException
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/
	public Collection getToDoListAllInstances() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection ready;
		try {
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			ready = nodehome.findUserReadyAllProjects(mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			ready = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = ready.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			if (nd.getBnProject().getName().matches(".*_instance.*"))
			  result.add(nd.getBnNodeValue());
		}
		log.info("End");
		return result;
	}
	
	/**
		* Obtain executing user activities for all instances (ready and anticipable state)
		* @return a BnNodeValue Collection - the list of executing activities ot the user for all instances
		* @throws HeroException
		* @ejb:interface-method view-type="both"
		* @ejb:transaction type="Supports"
		*
		**/
		public Collection getActivityListAllInstances() throws HeroException {
			log.info(" start by " + mContext.getCallerPrincipal().getName());
			BnNodeLocalHome nodehome;
			Collection executing;
			try {
				nodehome = BnNodeUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			}
			try {
				executing = nodehome.findUserExecutingAllProjects(mContext.getCallerPrincipal().getName());
			} catch (FinderException fe) {
				log.error(fe.getMessage());
				executing = new ArrayList();
			}
			ArrayList result = new ArrayList();
			for (Iterator i = executing.iterator(); i.hasNext();) {
				BnNodeLocal nd = (BnNodeLocal) i.next();
				if (nd.getBnProject().getName().matches(".*_instance.*"))
				  result.add(nd.getBnNodeValue());
			}
			log.info("End");
			return result;
		}
	
	/**
	* Obtain todo user activities matching with property value (ready and anticipable state activities)
	* @return a BnNodeValue Collection - the list of todo activities matching to property value
	* @param key the key of the property
	* @param value the value of the property
	* 
	* @throws HeroException
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/
	public Collection getToDoListByProperty(String key, String value) throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection ready;
		try {
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			ready = nodehome.findUserReadyAllProjectsByProperty(mContext.getCallerPrincipal().getName(),key,value);
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			ready = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = ready.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			if (nd.getBnProject().getName().matches(".*_instance.*"))
			  result.add(nd.getBnNodeValue());
		}
		log.info("End");
		return result;
	}
	
	/**
	* Obtain executing user activities matching with property value (executing and anticipating state activities)
	* @return a BnNodeValue Collection - the list of executing activities matching to property value
	* @param key the key of the property
	* @param value the value of the property
	* 
	* @throws HeroException
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/
	public Collection getActivityListByProperty(String key, String value) throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection executing;
		try {
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			executing = nodehome.findUserExecutingAllProjectsByProperty(mContext.getCallerPrincipal().getName(),key,value);
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			executing = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = executing.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			if (nd.getBnProject().getName().matches(".*_instance.*"))
			  result.add(nd.getBnNodeValue());
		}
		log.info("End");
		return result;
	}
	
	
	/**
	 * Get workflow instances names of this project
	 *
	 * @param projectName the name of the project
	 * @return a String Collection - the names of project instances
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProjectInstancesNames(String projectName)
		throws HeroException {
		log.info("parameter: projectName = "+ projectName+ " start by "+ mContext.getCallerPrincipal().getName());
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances =projhome.findInstancesByProject(projectName + "_instance%");
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				result.add(pl.getName());
			}
			log.info("End");
			return (result);
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}
	/**
	 * Get Workflow instances of this project
	 *
	 * @return a BnProjectValue Collection - list of project instances
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProjectInstances(String projectName) throws HeroException {
			log.info("parameter: projectName = "+projectName + " start by " + mContext.getCallerPrincipal().getName());
			ArrayList result = new ArrayList();
			BnProjectLocalHome projhome;
			try {
				projhome = hero.interfaces.BnProjectUtil.getLocalHome();
				Collection instances = projhome.findInstancesByProject(projectName+"_instance%");
				for (Iterator i = instances.iterator(); i.hasNext();) {
					BnProjectLocal pl = (BnProjectLocal) i.next();
					result.add(pl.getBnProjectValue());
				}
				log.info("End");
				return (result);
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			} catch (javax.ejb.FinderException fe) {
				log.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			}			
		}
		
	/**
	 * Get Workflow instances from a property
	 *
	 * @return a BnProjectValue Collection - list of project instances
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/

	public Collection getInstancesByProperty(String key, String value)
		throws HeroException {
		log.info("parameters: key = " + key +" value = " + value + " start by " + mContext.getCallerPrincipal().getName());
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findProjectsByProperty(key, value);
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				if (pl.getName().matches(".*_instance.*"))
					result.add(pl.getBnProjectValue());
			}
			log.info("End");
			return (result);
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}

	/**
	 * Get Workflow instances from a property
	 *
	 * @return a String Collection - list of project instances
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/

	public Collection getInstancesByPropertyNames(String key, String value)
		throws HeroException {
		log.info("parameters: key = " + key +" value = " + value + " start by " + mContext.getCallerPrincipal().getName());
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findProjectsByProperty(key, value);
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				if (pl.getName().matches(".*_instance.*"))
					result.add(pl.getName());

			}
			log.info("End");
			return (result);
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}

	/**
	* Get Workflow projects from a property
	*
	* @return a BnProjectValue Collection - list of projects
	* @param projectName the name of the project
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/

	public Collection getProjectsByProperty(String key, String value)
		throws HeroException {
		log.info("parameters: key = " + key +" value = " + value + " start by " + mContext.getCallerPrincipal().getName());
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findProjectsByProperty(key, value);
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				if (!pl.getName().matches(".*_instance.*"))
					result.add(pl.getBnProjectValue());
			}
			log.info("End");
			return (result);
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}

	/**
	 * Get Workflow projects from a property
	 *
	 * @return a String Collection - list of project names
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	
	public Collection getProjectsByPropertyNames(String key, String value)
		throws HeroException {
		log.info("parameters: key = " + key +" value = " + value + " start by " + mContext.getCallerPrincipal().getName());
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findProjectsByProperty(key, value);
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				if (!pl.getName().matches(".*_instance.*"))
					result.add(pl.getName());

			}
			log.info("End");
			return (result);
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}

	/**
		 * Tries to start an activity (when activity state is ready or anticipable)
		 * @param projectName the name of the project
		 * @param nodeName the name of the node
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Required"
		 **/

		public void startActivity(String projectName, String nodeName)
			throws HeroException {
			log.info("parameters: projectName="+ projectName+ " nodeName="+ nodeName + " start by " + mContext.getCallerPrincipal().getName());
			EngineLocalHome enghome;
			try {
				enghome = hero.interfaces.EngineUtil.getLocalHome();
				EngineLocal engine = enghome.create();
				engine.startActivity(projectName, nodeName);
				engine.remove();
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			} catch (RemoveException r) {
				log.error("Cannot remove Engine bean for "+ projectName+ " " + r.getMessage());
				throw new HeroException("Cannot remove Engine bean for "+ projectName+ " "+ r.getMessage());
			} catch (CreateException c) {
				log.error("Cannot create Engine bean for "+ projectName+ " " + c.getMessage());
				throw new HeroException("Cannot create Engine bean for "+ projectName+ " "+ c.getMessage());
			}
			log.info("End");
		}

		/**
		 * Tries to terminate an activity (when activity state is executing or anticipating)
		 * @param projectName the name of the project
		 * @param nodeName the name of the node
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Required"
		 **/
		public void terminateActivity(String projectName, String nodeName)
			throws HeroException {
			log.info("parameters: projectName="+ projectName+ " nodeName="+ nodeName + " start by " + mContext.getCallerPrincipal().getName());
			EngineLocalHome enghome;
			try {
				enghome = hero.interfaces.EngineUtil.getLocalHome();
				EngineLocal engine = enghome.create();
				engine.terminateActivity(projectName, nodeName);
				engine.remove();
			} catch (RemoveException r) {
				log.error("Cannot remove Engine bean for "+ projectName+ " " + r.getMessage());
				throw new HeroException("Cannot remove Engine bean for "+ projectName+ " "+ r.getMessage());			
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			} catch (CreateException c) {
				log.error("Cannot create Engine bean for " + projectName + " " + c.getMessage());
				throw new HeroException("Cannot create Engine bean for " + projectName);
			}
			log.info("End");
		}

		/**
		 * Tries to cancel an activity (when activity is executing or anticipating)
		 * @param projectName the name of the project
		 * @param nodeName the name of the node
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Required"
		 **/
		public void cancelActivity(String projectName, String nodeName)
			throws HeroException {
			log.info("parameters: projectName="+ projectName+ " nodeName="+ nodeName + " start by " + mContext.getCallerPrincipal().getName());
			EngineLocalHome enghome;
			try {
				enghome = hero.interfaces.EngineUtil.getLocalHome();
				EngineLocal engine = enghome.create();
				engine.cancelActivity(projectName, nodeName);
				engine.remove();
			} catch (RemoveException r) {
				log.error("Cannot remove Engine bean for "+ projectName + " " + r.getMessage());
				throw new HeroException("Cannot remove Engine bean for "+ projectName+ " "+ r.getMessage());
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			} catch (CreateException c) {
				log.error("Cannot create Engine bean for " + projectName + " " + c.getMessage());
				throw new EJBException("Cannot create Engine bean for " + projectName);
			}
			log.info("End");
		}
		/**
		 * Tries to terminate a project (only when all project activities are terminated)
		 * @param projectName the name of the project
		 * @throws HeroException
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Required"
		 **/
		public void terminate(String projectName) throws HeroException {
			log.info("parameter; projectName=" + projectName + " start by " + mContext.getCallerPrincipal().getName());
			EngineLocalHome enghome;
			try {
				enghome = hero.interfaces.EngineUtil.getLocalHome();
				EngineLocal engine = enghome.create();
				engine.terminate(projectName);
				engine.remove();
			} catch (RemoveException r) {
				log.error("Cannot remove Engine bean for "+ projectName+ " " + r.getMessage());
				throw new HeroException("Cannot remove Engine bean for "+ projectName+ " "+ r.getMessage());
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			} catch (CreateException c) {
				log.error("Cannot create Engine bean for " + projectName + " " + c.getMessage());
				throw new EJBException("Cannot create Engine bean for " + projectName);
			} catch (EngineException e) {
				log.error("Cannot terminate project " + e.getMessage());
				throw new HeroException("Cannot terminate project " + e.getMessage());
			}
			log.info("End");
		}

		/**
		 * Get Node Value from a specific project
		 * @return a NodeValue object - get node information
		 * @param projectName the name of the project
		 * @param nodeName the name of the node
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
		 **/
		public BnNodeValue getNode(String projectName, String nodeName)
			throws HeroException {
			log.info("parameters: projectName=" + projectName + " nodeName=" + nodeName + "start by " + mContext.getCallerPrincipal().getName());
			try {
				ProjectSessionLocalHome projectseshome =ProjectSessionUtil.getLocalHome();
				ProjectSessionLocal project = projectseshome.create();
				project.initProject(projectName);
				if (!checkRights(project))
					throw new HeroException("User is not member of the project");
				log.info("End");
				return project.getNodeValue(nodeName);
			} catch (CreateException f) {
				log.error("Project does not exists " + projectName + " " + f.getMessage());
				throw new HeroException("Project does not exists " + projectName);
			} catch (javax.naming.NamingException r) {
				log.error("Cannot access to project " + r.getMessage());
				throw new HeroException("Cannot access to project " + r.getMessage());
			}
		}

		/**
		 * Get the user name
		 * 
		 * @return a String - get user name
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
		 **/
		public String getUser() {
			log.info("start by " + mContext.getCallerPrincipal().getName());
			log.info("End");
			return mContext.getCallerPrincipal().getName();
		}

		/**
		 * Get the user light value
		 * 
		 * @return a BnUserLightValue object - get user information
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
		 **/
		public BnUserLightValue getUserLightValue() throws HeroException{
			try{
				log.info("start by " + mContext.getCallerPrincipal().getName());
				BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
				log.info("End");
				return mUser.getBnUserLightValue();
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			} catch (javax.ejb.FinderException fe) {
				log.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
				throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
			}
		}
	
		/**
		 * Get the mail of this user from Bonita database.
		 *
		 * @param userName the name of the user
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
		 * @throws HeroException
		 *
		**/
		public String getUserMail(String userName) throws HeroException {
			log.info("parameter: userName = " + userName + " start by " + mContext.getCallerPrincipal().getName());
			try {
				BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal user = userhome.findByName(userName);
				log.info("End");
				return (user.getEmail());
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
					throw new HeroException(ne.getMessage());
			} catch (FinderException fe) {
				log.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			}
		}
	
		/**
		 * Set the mail of this user into Bonita database.
		 *
		 * @param userName the name of the user
		 * @param userName the mail of the user
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Required"
		 * @throws HeroException
		 *
		**/
		public void setUserMail(String userName, String mail) throws HeroException {
			log.info("parameters:: userName = " + userName+" mail = "+mail + " start by " + mContext.getCallerPrincipal().getName());
			try {
				BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal user = userhome.findByName(userName);
				user.setEmail(mail);
				log.info("End");
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			} catch (FinderException fe) {
				log.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			}
		}
	

		/**
		 * Get the user password
		 *
		 * @return a String - get user password
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
		 **/
		public String getUserPassword() throws HeroException{
			try{
				log.info("start by " + mContext.getCallerPrincipal().getName());
				BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
				log.info("End");
				return mUser.getPassword();
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			} catch (javax.ejb.FinderException fe) {
				log.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
				throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
			}
		}

		/**
		  * Get User properties
		  *
		  * @return a BnUserPropertyValue Collection - list of user properties
		  * @ejb:interface-method view-type="both"
		  * @ejb:transaction type="Supports"
		  *
		  **/
		public Collection getUserProperties() throws HeroException {
			Collection prp = new ArrayList();
			BnUserPropertyLocal pl;
			log.info("start by " + mContext.getCallerPrincipal().getName());
			try{
				BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
				Collection pls = mUser.getBnProperties();

				for (Iterator i = pls.iterator(); i.hasNext();) {
					pl = (BnUserPropertyLocal) i.next();
					prp.add(pl.getBnUserPropertyValue());
				}
				log.info("End");
				return (prp);
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			} catch (javax.ejb.FinderException fe) {
				log.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
				throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
			}
		}

		/**
		 * Set a property of a user
		 *
		 * @param key the key of the property
		 * @param value the name of the property
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Required"
		 *
		 **/

		public void setUserProperty(String key, String value)
			throws HeroException {
			BnUserPropertyLocalHome pHome;
			BnUserPropertyLocal propertyLocal = null;

			log.info("parameters:  key=" + key + " value=" + value + " start by " + mContext.getCallerPrincipal().getName());

			try {
				BnUserLocalHome userhome =	hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal mUser =	userhome.findByName(mContext.getCallerPrincipal().getName());
				try {
					pHome = BnUserPropertyUtil.getLocalHome();
				} catch (javax.naming.NamingException ne) {
					log.error(ne.getMessage());
					throw new HeroException(ne.getMessage());
				}
				Collection c = null;
				try {
					c = pHome.findByTheKey(mUser.getId(), key);
				} catch (FinderException fe) {
					log.error(fe.getMessage());
					throw new HeroException(fe.getMessage());
				}

				if (!c.isEmpty()) {

					propertyLocal = (BnUserPropertyLocal) (c.toArray())[0];
					propertyLocal.setTheValue(value);

				} else {
					try {
						propertyLocal = pHome.create(key, value);
						propertyLocal.setBnUser(mUser);

					} catch (InvalidValueException ie) {
						log.error(ie.getMessage());
						throw new HeroException(ie.getMessage());
					} catch (javax.ejb.CreateException ce) {
						log.error(ce.getMessage());
						throw new HeroException(ce.getMessage());
					}

				}
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			} catch (javax.ejb.FinderException fe) {
				log.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
				throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
			}
		}

		/**
		  * Delete a property of a user
		  *
		  * @param key the key of the property
		  * @ejb:interface-method view-type="both"
		  * @ejb:transaction type="Required"
		  *
		  **/
		public void deleteUserProperty(String key) throws HeroException {
			log.info("parameter key=" + key + " start by " + mContext.getCallerPrincipal().getName());

			BnUserPropertyLocalHome pHome;
			BnUserPropertyLocal propertyLocal = null;
			try {
				BnUserLocalHome userhome =	hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal mUser =	userhome.findByName(mContext.getCallerPrincipal().getName());
				try {
					pHome = BnUserPropertyUtil.getLocalHome();
				} catch (javax.naming.NamingException ne) {
					log.error(ne.getMessage());
					throw new HeroException(ne.getMessage());
				}
				Collection c = null;
				try {
					c = pHome.findByTheKey(mUser.getId(), key);
				} catch (FinderException fe) {
					log.error(fe.getMessage());
					throw new HeroException(fe.getMessage());
				}

				if (!c.isEmpty()) {
					try {
						propertyLocal = (BnUserPropertyLocal) (c.toArray())[0];
						propertyLocal.remove();
					} catch (RemoveException rm) {
						log.error(rm.getMessage());
						throw new HeroException(rm.getMessage());
					}
				} else {
					throw new HeroException("Property not found :" + key);
				}

			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			} catch (javax.ejb.FinderException fe) {
				log.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
				throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
			}
		log.info("End");
		}

	/**
	* Creates the User Session Bean. This method is the first one to invoke in order to
	* use UserSession API. If the user is not authorized this method throws an exception.
	*
	* @throws CreateException 
	*
	* @ejb:create-method view-type="both"
	**/
	public void ejbCreate() throws CreateException {
	}

	/**
    * Internal Enterprise Java Beans method. 
	**/

	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
	}

	/**
	* Internal Enterprise Java Beans method. 
	**/
	public void ejbRemove() {
	}

	/**
	* Internal Enterprise Java Beans method. 
	**/
	public void ejbActivate() {
	}

	/**
	* Internal Enterprise Java Beans method. 
	**/
	public void ejbPassivate() {
	}

	// utility methods
	private boolean checkRights(ProjectSessionLocal ps) throws HeroException {
		// check that current user is a member of the project
		return ps.containsUser(mContext.getCallerPrincipal().getName());
	}
}
